# -*- coding: utf-8 -*-
"""
General exceptions; not overly module-specific
"""
__author__ = "Zack Cerza <zcerza@redhat.com>"
from logging import debugLogger as logger

import inspect
def warn(message, caller = True):
    """
    Generate a warning, and pass it to the debug logger.
    """
    frameRec = inspect.stack()[-1]
    message = "Warning: %s:%s: %s" % (frameRec[1], frameRec[2], message)
    if caller and frameRec[1] != '<stdin>' and frameRec[1] != '<string>': 
        message = message + ':\n  ' + frameRec[4][0]
    del frameRec
    logger.log(message)

class DependencyNotFoundError(Exception):
    """
    A dependency was not found.
    """
    pass




"""Handles versioning of software packages

Author: Dave Malcolm <dmalcolm@redhat.com>"""
__author__ = 'Dave Malcolm <dmalcolm@redhat.com>'

import re
import unittest

class Version:
    """
    Class representing a version of a software package.
    Stored internally as a list of subversions, from major to minor.
    Overloaded comparison operators ought to work sanely.
    """
    def __init__(self, versionList):
        self.versionList = versionList

    def fromString(versionString):
        """
        Parse a string of the form number.number.number
        """
        return Version(map(int, versionString.split(".")))
    fromString = staticmethod(fromString)

    def __str__(self):
        return ".".join(map(str, self.versionList))

    def __getNum(self):
        tmpList = list(self.versionList)

        while len(tmpList)<5:
            tmpList += [0]

        num = 0
        for i in range(len(tmpList)):
            num *=1000
            num += tmpList[i]
        return num

    def __lt__(self, other):
        return self.__getNum()<other.__getNum()
    def __le__(self, other):
        return self.__getNum()<=other.__getNum()
    def __eq__(self, other):
        return self.__getNum()==other.__getNum()
    def __ne__(self, other):
        return self.__getNum()!=other.__getNum()
    def __gt__(self, other):
        return self.__getNum()>other.__getNum()
    def __ge__(self, other):
        return self.__getNum()>=other.__getNum()

class VersionTests(unittest.TestCase):
    def testLt(self):
        assert Version.fromString("2.0.1") < Version.fromString("2.1.0")
        assert not Version.fromString("1.4.0") < Version.fromString("1.4.0")

    def testLe(self):
        assert Version.fromString("2.0.1") <= Version.fromString("2.1.0")
        assert Version.fromString("1.4.0") <= Version.fromString("1.4.0")

    def testEq(self):
        assert not Version.fromString("2.0.1") == Version.fromString("2.1.0")
        assert Version.fromString("1.4.0") == Version.fromString("1.4.0")

    def testNe(self):
        assert Version.fromString("2.0.1") != Version.fromString("2.1.0")
        assert not Version.fromString("1.4.0") != Version.fromString("1.4.0")

    def testGt(self):
        assert Version.fromString("2.1.0") > Version.fromString("2.0.1")
        assert not Version.fromString("1.4.0") > Version.fromString("1.4.0")

    def testGe(self):
        assert Version.fromString("2.1.0") >= Version.fromString("2.0.1")
        assert Version.fromString("1.4.0") >= Version.fromString("1.4.0")

    def testStr(self):
        assert "2.0.4"==str(Version.fromString("2.0.4"))

    def testParsing(self):
        assert Version.fromString("0") == Version([0])
        assert Version.fromString("0.1") == Version([0, 1])
        assert Version.fromString("1.4.0") == Version([1, 4, 0])

if __name__ == "__main__":
    unittest.main()



"""Handles differences between different distributions

Authors: Dave Malcolm <dmalcolm@redhat.com>, Zack Cerza <zcerza@redhat.com>"""
__author__ = "Dave Malcolm <dmalcolm@redhat.com>, Zack Cerza <zcerza@redhat.com>"

import os
import re
from version import Version
from logging import debugLogger as logger

class DistributionNotSupportedError(Exception):
    """
    This distribution is not supported.
    """
    PATCH_MESSAGE = "Please send patches to dogtail-devel-list@gnome.org"

    def __init__(self, distro):
        self.distro = distro

    def __str__(self):
        return self.distro + ". " + DistributionNotSupportedError.PATCH_MESSAGE

class PackageNotFoundError(Exception):
    """
    Error finding the requested package.
    """
    pass

global packageDb
global distro

class PackageDb:
    """
    Class to abstract the details of whatever software package database is in
    use (RPM, APT, etc)
    """
    def __init__(self):
        self.prefix = '/usr'
        self.localePrefixes = [self.prefix + '/share/locale']

    def getVersion(self, packageName):
        """
        Method to get the version of an installed package as a Version 
        instance (or raise an exception if not found)
        
        Note: does not know about distributions' internal revision numbers.
        """
        raise NotImplementedError

    def getFiles(self, packageName):
        """
        Method to get a list of filenames owned by the package, or raise an 
        exception if not found.
        """
        raise NotImplementedError

    def getMoFiles(self, locale = None):
        """
        Method to get a list of all .mo files on the system, optionally for a
        specific locale.
        """
        moFiles = {}

        def appendIfMoFile(moFiles, dirName, fNames):
            import re
            for fName in fNames:
                if re.match('(.*)\\.mo', fName):
                    moFiles[dirName + '/' + fName] = None

        for localePrefix in self.localePrefixes:
            if locale: localePrefix = localePrefix + '/' + locale
            os.path.walk(localePrefix, appendIfMoFile, moFiles)

        return moFiles.keys()

    def getDependencies(self, packageName):
        """
        Method to get a list of unique package names that this package
        is dependent on, or raise an exception if the package is not
        found.
        """
        raise NotImplementedError

class _RpmPackageDb(PackageDb):
    def __init__(self):
        PackageDb.__init__(self)

    def getVersion(self, packageName):
        import rpm
        ts = rpm.TransactionSet()
        for header in ts.dbMatch("name", packageName):
            return Version.fromString(header["version"])
        raise PackageNotFoundError, packageName

    def getFiles(self, packageName):
        import rpm
        ts = rpm.TransactionSet()
        for header in ts.dbMatch("name", packageName):
            return header["filenames"]
        raise PackageNotFoundError, packageName

    def getDependencies(self, packageName):
        import rpm
        ts = rpm.TransactionSet()
        for header in ts.dbMatch("name", packageName):
            # Simulate a set using a hash (to a dummy value);
            # sets were only added in Python 2.4
            result = {}

            # Get the list of requirements; these are
            # sometimes package names, but can also be
            # so-names of libraries, and invented virtual
            # ids
            for requirement in header[rpm.RPMTAG_REQUIRES]:
                # Get the name of the package providing
                # this requirement:
                for depPackageHeader in ts.dbMatch("provides", requirement):
                    depName = depPackageHeader['name']
                    if depName!=packageName:
                        # Add to the Hash with a dummy value
                        result[depName]=None
            return result.keys()
        raise PackageNotFoundError, packageName

class _AptPackageDb(PackageDb):
    def __init__(self):
        PackageDb.__init__(self)
        self.cache = None

    def getVersion(self, packageName):
        if not self.cache:
            import apt_pkg
            apt_pkg.init()
            self.cache = apt_pkg.GetCache()
        packages = self.cache.Packages
        for package in packages:
            if package.Name == packageName:
                import re
                verString = re.match('.*Ver:\'(.*)-.*\' Section:', str(package.CurrentVer)).group(1)
                return Version.fromString(verString)
        raise PackageNotFoundError, packageName

    def getFiles(self, packageName):
        files = []
        list = os.popen('dpkg -L %s' % packageName).readlines()
        if not list:
            raise PackageNotFoundError, packageName
        else:
            for line in list:
                file = line.strip()
                if file: files.append(file)
            return files

    def getDependencies(self, packageName):
        # Simulate a set using a hash (to a dummy value);
        # sets were only added in Python 2.4
        result = {}
        if not self.cache:
            import apt_pkg
            apt_pkg.init()
            self.cache = apt_pkg.GetCache()
        packages = self.cache.Packages
        for package in packages:
            if package.Name == packageName:
                current = package.CurrentVer
                if not current:
                    raise PackageNotFoundError, packageName
                depends = current.DependsList
                list = depends['Depends']
                for dependency in list:
                    name = dependency[0].TargetPkg.Name
                    # Add to the hash using a dummy value
                    result[name] = None
        return result.keys()

class _UbuntuAptPackageDb(_AptPackageDb):
    def __init__(self):
        _AptPackageDb.__init__(self)
        self.localePrefixes.append(self.prefix + '/share/locale-langpack')

class _PortagePackageDb(PackageDb):
    def __init__(self):
        PackageDb.__init__(self)

    def getVersion(self, packageName):
        # the portage utilities are almost always going to be in 
        # /usr/lib/portage/pym
        import sys
        sys.path.append ('/usr/lib/portage/pym')
        import portage
        # FIXME: this takes the first package returned in the list, in the 
        # case that there are slotted packages, and removes the leading 
        # category such as 'sys-apps'
        gentooPackageName = portage.db["/"]["vartree"].dbapi.match(packageName)[0].split('/')[1];
        # this removes the distribution specific versioning returning only the
        # upstream version
        upstreamVersion = portage.pkgsplit(gentooPackageName)[1]
        #print "Version of package is: " + upstreamVersion
        return Version.fromString(upstreamVersion);

class _ConaryPackageDb(PackageDb):
    def __init__(self):
        PackageDb.__init__(self)

    def getVersion(self, packageName):
        import conary
        from conaryclient import ConaryClient
        client = ConaryClient()
        dbVersions = client.db.getTroveVersionList(packageName)
        if not len(dbVersions):
            raise PackageNotFoundError, packageName
        return dbVersions[0].trailingRevision().asString().split("-")[0]

# getVersion not implemented because on Solaris multiple modules are installed
# in single packages, so it is hard to tell what version number of a specific
# module.
class _SolarisPackageDb(PackageDb):
    def __init__(self):
        PackageDb.__init__(self)

class JhBuildPackageDb(PackageDb):
    def __init__(self):
        PackageDb.__init__(self)
        prefixes = []
        prefixes.append(os.environ['LD_LIBRARY_PATH'])
        prefixes.append(os.environ['XDG_CONFIG_DIRS'])
        prefixes.append(os.environ['PKG_CONFIG_PATH'])
        self.prefix = os.path.commonprefix(prefixes)
        self.localePrefixes.append(self.prefix + '/share/locale')

    def getDependencies(self, packageName):
        result = {}
        lines = os.popen('jhbuild list ' + packageName).readlines()
        for line in lines:
            if line:
                result[line.strip()] = None
        return result.keys()

class Distro:
    """
    Class representing a distribution.

    Scripts may want to do arbitrary logic based on whichever distro is in use
    (e.g. handling differences in names of packages, distribution-specific 
    patches, etc.)

    We can either create methods in the Distro class to handle these, or we 
    can use constructs like isinstance(distro, Ubuntu) to handle this. We can
    even create hierarchies of distro subclasses to handle this kind of thing
    (could get messy fast though)
    """

class Fedora(Distro):
    def __init__(self):
        self.packageDb = _RpmPackageDb()

class RHEL(Fedora):
    pass

class Debian(Distro):
    def __init__(self):
        self.packageDb = _AptPackageDb()

class Ubuntu(Debian):
    def __init__(self):
        self.packageDb = _UbuntuAptPackageDb()

class Suse(Distro):
    def __init__(self):
        self.packageDb = _RpmPackageDb()

class Gentoo(Distro):
    def __init__(self):
        self.packageDb = _PortagePackageDb()

class Conary(Distro):
    def __init__(self):
        self.packageDb = _ConaryPackageDb()

class Solaris(Distro):
    def __init__(self):
        self.packageDb = _SolarisPackageDb()

class JHBuild(Distro):
    def __init__(self):
        self.packageDb = JhBuildPackageDb()

def detectDistro():
    logger.log("Detecting distribution:", newline = False)

    if os.environ.get("CERTIFIED_GNOMIE", "no") == "yes":
        distro = JHBuild()
    elif os.path.exists("/etc/SuSE-release"):
        distro = Suse()
    elif os.path.exists("/etc/fedora-release"):
        distro = Fedora()
    elif os.path.exists("/etc/redhat-release"):
        distro = RHEL()
    elif os.path.exists("/usr/share/doc/ubuntu-minimal"):
        distro = Ubuntu()
    elif os.path.exists("/etc/debian_version"):
        distro = Debian()
    elif os.path.exists("/etc/gentoo-release"):
        distro = Gentoo()
    elif os.path.exists("/etc/slackware-version"):
        raise DistributionNotSupportedError("Slackware")
    elif os.path.exists("/var/lib/conarydb/conarydb"):
        distro = Conary()
    elif os.path.exists ("/etc/release") and \
            re.match(".*Solaris", open ("/etc/release").readline()):
        distro = Solaris()
    else:
        raise DistributionNotSupportedError("Unknown")
    logger.log(distro.__class__.__name__)
    return distro

distro = detectDistro()
packageDb = distro.packageDb




#!/usr/bin/python
"""
Unit tests for the dogtail.procedural API
"""
__author__ = "Zack Cerza <zcerza@redhat.com>"

import unittest
import dogtail.tree
import dogtail.predicate
dogtail.config.config.logDebugToFile = False
dogtail.config.config.logDebugToStdOut = True
import Node


class TestScreenshot(Node.GtkDemoTest):

    def make_expected_and_compare(self, actual_path, jpg_tolerance=None):
        extension = actual_path.split('.')[-1]
        expected_path = actual_path.replace(extension, "expected." + extension)

        import os
        os.system("gnome-screenshot -f %s" % expected_path)

        command = ["compare", "-metric", "MAE", actual_path, expected_path, "output"]
        import subprocess
        p = subprocess.Popen(command, stderr=subprocess.PIPE)
        output, error = p.communicate()

        import re
        m = re.search(r"\((.*)\)", error)
        self.assertTrue(0.1 >= float(m.group(1)))

    def test_screenshot_incorrect_timestamp(self):
        self.assertRaises(TypeError, dogtail.utils.screenshot, "timeStamp", None)

    def test_screenshot_default(self):
        actual_path = dogtail.utils.screenshot()
        self.make_expected_and_compare(actual_path)

    def test_screenshot_basename(self):
        actual_path = dogtail.utils.screenshot("basename")
        self.make_expected_and_compare(actual_path)

    def test_screenshot_no_time_stamp(self):
        actual_path = dogtail.utils.screenshot(timeStamp=False)
        self.make_expected_and_compare(actual_path)

    def test_screenshot_jpeg(self):
        actual_path = dogtail.utils.screenshot("basename.jpg")
        self.make_expected_and_compare(actual_path, jpg_tolerance=True)

    def test_screenshot_unknown_format(self):
        self.assertRaises(ValueError, dogtail.utils.screenshot, "basename.dat")


class TestA11Y(unittest.TestCase):

    def test_bail_when_a11y_disabled(self):
        self.assertRaises(SystemExit, dogtail.utils.bailBecauseA11yIsDisabled)

    def test_enable_a11y(self):
        dogtail.utils.enableA11y()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Dogtail demo script
# Note that this script is outdated and may cause your session to lock up until
# it is killed. It will most likely be deleted in the next release.
__author__ = 'David Malcolm <dmalcolm@redhat.com>'

import dogtail.tree
import pyatspi
import Accessibility

# Test of event callbacks
# Under construction

def callback(event):
    source = event.source
    if isinstance(source, Accessibility.Accessible):
        sourceStr = " source:%s"% str(source)
    else:
        sourceStr = ""
    print "Got event: %s%s"%(event.type, sourceStr)

#listener = atspi.EventListener(callback, ["window:create"])
#listener = atspi.EventListener(callback, ["focus:", "object:", "window:"])
#listener = atspi.EventListener(callback, ["window:"])
#listener = atspi.EventListener(callback, ["object:"])
#listener = atspi.EventListener(callback, ["focus:"])

# explicit list of all events, taken from at-spi/test/event-listener-test.c:
eventNames = [
    "focus:",
    "mouse:rel",
    "mouse:button",
    "mouse:abs",
    "keyboard:modifiers",
    "object:property-change",
    "object:property-change:accessible-name",
    "object:property-change:accessible-description",
    "object:property-change:accessible-parent",
    "object:state-changed",
    "object:state-changed:focused",
    "object:selection-changed",
    "object:children-changed",
    "object:active-descendant-changed",
    "object:visible-data-changed",
    "object:text-selection-changed",
#    "object:text-caret-moved",
#    "object:text-changed",
    "object:column-inserted",
    "object:row-inserted",
    "object:column-reordered",
    "object:row-reordered",
    "object:column-deleted",
    "object:row-deleted",
    "object:model-changed",
    "object:link-selected",
    #"object:bounds-changed", # avoid swamping log
    "window:minimize",
    "window:maximize",
    "window:restore",
    "window:activate",
    "window:create",
    "window:deactivate",
    "window:close",
    "window:lower",
    "window:raise",
    "window:resize",
    "window:shade",
    "window:unshade",
    "object:property-change:accessible-table-summary",
    "object:property-change:accessible-table-row-header",
    "object:property-change:accessible-table-column-header",
    "object:property-change:accessible-table-summary",
    "object:property-change:accessible-table-row-description",
    "object:property-change:accessible-table-column-description",
    "object:test"
    ]

listeners = []
for eventName in eventNames:
    #listener = atspi.EventListener(callback, [eventName])
    listeners.append(pyatspi.Registry.registerEventListener(callback, eventName))

#listener = atspi.EventListener(callback, [""])
pyatspi.Registry.start(False, False)



# -*- coding: UTF-8 -*-
"""
GUI test tool and automation framework that uses Accessibility (a11y) technologies to communicate with desktop applications.

Authors: Zack Cerza <zcerza@redhat.com>, Ed Rousseau <rousseau@redhat.com>, David Malcolm <dmalcolm@redhat.com>, Vita Humpa <vhumpa@redhat.com>
"""

__author__ = """Zack Cerza <zcerza@redhat.com>,
Ed Rousseau <rousseau@redhat.com>,
David Malcolm <dmalcolm@redhat.com>,
Vita Humpa <vhumpa@redhat.com>"""
__version__ = "0.8.2"
__copyright__ = "Copyright © 2005-2012 Red Hat, Inc."
__license__ = "GPL"

import gi.repository
__all__ = ("config", "predicate", "procedural", "tc", "tree", "utils", "errors")



"""Utility functions for 'dumping' trees of Node objects.

Author: Zack Cerza <zcerza@redhat.com>"""
__author__ = "Zack Cerza <zcerza@redhat.com>"

spacer = ' '
def plain (node, fileName = None):
    """
    Plain-text dump. The hierarchy is represented through indentation.
    """
    def crawl(node, depth):
        dump(node, depth)
        for action in node.actions.values():
            dump(action, depth + 1)
        for child in node.children:
            crawl(child, depth + 1)

    def dumpFile(item, depth): _file.write(spacer*depth + str(item) + '\n')
    def dumpStdOut(item, depth): print spacer*depth + str(item)
    if fileName:
        dump = dumpFile
        _file = file(fileName, 'w')
    else: dump = dumpStdOut
    
    crawl(node, 0)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unit tests for the dogtail.Node class

Notes on pyunit (the "unittest" module):

Test classes are written as subclass of unittest.TestCase.
A test is a method of such a class, beginning with the string "test"

unittest.main() will run all such methods.  Use "-v" to get feedback on which tests are being run.  Tests are run in alphabetical order; all failure reports are gathered at the end.

setUp and tearDown are "magic" methods, called before and after each such
test method is run.
"""
__author__="Dave Malcolm <dmalcolm@redhat.com>"

import unittest
import time
import dogtail.tree
import dogtail.predicate
import dogtail.config
dogtail.config.config.logDebugToFile = False
import pyatspi

class GtkDemoTest(unittest.TestCase):
    """
    TestCase subclass which handles bringing up and shutting down gtk-demo as a fixture.  Used for writing other test cases.
    """
    def setUp(self):
        import dogtail.utils
        self.pid = dogtail.utils.run('gtk-demo')
        self.app = dogtail.tree.root.application('gtk-demo')

    def tearDown(self):
        import os, signal
        os.kill(self.pid, signal.SIGKILL)
        # Sleep just enough to let the app actually die.
        # AT-SPI doesn't like being hammered too fast.
        time.sleep(0.5)

    def runDemo(self, demoName):
        """
        Click on the named demo within the gtk-demo app.
        """
        tree = self.app.child(roleName="tree table")
        tree.child(demoName).doubleClick()

class TestNodeAttributes(GtkDemoTest):
    """
    Unit tests for the the various synthesized attributes of a Node
    """
    def testGetBogus(self):
        "Getting a non-existant attribute should raise an attribute error"
        self.assertRaises(AttributeError, getattr, self.app, "thisIsNotAnAttribute")

    #FIXME: should setattr for a non-existant attr be allowed?

    # 'name' (read-only string):
    def testGetName(self):
        """
        Node.name of the gtk-demo app should be "gtk-demo"
        """
        self.assertEquals(self.app.name, 'gtk-demo')

        self.assertEquals(dogtail.tree.root.name, 'main')

    def testSetName(self):
        "Node.name should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "name", "hello world")

    # 'roleName' (read-only string):
    def testGetRoleName(self):
        """
        roleName of the gtk-demo app should be "application"
        """
        self.assertEquals(self.app.roleName, 'application')

    def testSetRoleName(self):
        "Node.roleName should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "roleName", "hello world")

    # 'role' (read-only atspi role enum):
    def testGetRole(self):
        "Node.role for a gtk-demo app should be SPI_ROLE_APPLICATION"
        self.assertEquals(self.app.role, dogtail.tree.pyatspi.ROLE_APPLICATION)

    def testSetRole(self):
        "Node.role should be read-only"
        # FIXME should be AttributeError?
        self.assertRaises(RuntimeError, self.app.__setattr__,  "role", pyatspi.Atspi.Role(1))

    # 'description' (read-only string):
    def testGetDescription(self):
        # FIXME: can we get a more interesting test case here?
        self.assertEquals(self.app.description, "")

    def testSetDescription(self):
        "Node.description should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "description", "hello world")

    # 'parent' (read-only Node instance):
    def testGetParent(self):
        # the app has a parent if gnome-shell is used, so parent.parent is a safe choice
        if filter(lambda x: x.name == 'gnome-shell', self.app.applications()):
            self.assertEquals(self.app.parent.parent, None)
        self.assertEquals(self.app.children[0].parent, self.app)

    def testSetParent(self):
        "Node.parent should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "parent", None)

    # 'children' (read-only list of Node instances):
    def testGetChildren(self):
        "A fresh gtk-demo app should have a single child: the window."
        kids = self.app.children
        self.assertEquals(len(kids), 1)
        self.assertEquals(kids[0].name, "GTK+ Code Demos")
        self.assertEquals(kids[0].roleName, "frame")

    def testSetChildren(self):
        "Node.children should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "children", [])

    # 'text' (string):
    def testSimpleTextEntry(self):
        """
        Use gtk-demo's text entry example to check that reading and writing
        Node.text works as expected
        """
        self.runDemo('Dialog and Message Boxes')
        wnd = self.app.window('Dialogs')
        wnd.button('Interactive Dialog').click()
        dlg = self.app.dialog('Interactive Dialog')
        entry1 = dlg.child(label='Entry 1')
        entry2 = dlg.child(label='Entry 2')

        # Try reading the entries:
        self.assertEquals(entry1.text, "")
        self.assertEquals(entry2.text, "")

        # Set them...
        entry1.text = "hello"
        entry2.text = "world"

        # Ensure that they got set:
        self.assertEquals(entry1.text, "hello")
        self.assertEquals(entry2.text, "world")

        # and try again, searching for them again, to ensure it actually affected the UI:
        self.assertEquals(dlg.child(label='Entry 1').text, "hello")
        self.assertEquals(dlg.child(label='Entry 2').text, "world")

        # Ensure app.text is None
        self.assertEquals(self.app.text, None)

        # Ensure a label's text is read-only as expected:
        # FIXME: this doesn't work; the label has no 'text'; it has a name.  we wan't a readonly text entry
        # label = dlg.child('Entry 1')
        # self.assertRaises(dogtail.tree.ReadOnlyError, label.text.__setattr__,  "text", "hello world")

        # FIXME: should we assert that things are logged and delays are added?
        # FIXME: should have a test case involving the complex GtkTextView widget

    def testCaretOffset(self):
        "Make sure the caret offset works as expected"
        self.runDemo('Dialog and Message Boxes')
        wnd = self.app.window('Dialogs')
        entry1 = wnd.child(label = 'Entry 1')
        entry2 = wnd.child(label = 'Entry 2')

        # Try reading the entries:
        self.assertEquals(entry1.text,'')
        self.assertEquals(entry2.text,'')

        # Set them...
        s1 = "I just need a sentence"
        s2 = "And maybe a second one to be sure"
        entry1.text = s1
        entry2.text = s2

        # Make sure the caret offset is zero
        self.assertEquals(entry1.caretOffset, 0)
        self.assertEquals(entry2.caretOffset, 0)

        # Set the caret offset to something ridiculous
        entry1.caretOffset = len(s1 * 3)
        entry2.caretOffset = len(s2 * 3)

        # Make sure the caret offset only goes as far as the end of the string
        self.assertEquals(entry1.caretOffset, len(s1))
        self.assertEquals(entry2.caretOffset, len(s2))

        def splitByOffsets(node, string):
            # Verify the equality of node.text and string, word by word.
            # I realize this doesn't really test dogtail itself, but that could
            #   change in the future and I don't want to throw the code away.
            textIface = node.queryText()
            endOffset = -1 # We only set this now so the loop looks nicer
            startOffset = 0
            while startOffset != len(string):
                (text, startOffset, endOffset) = textIface.getTextAtOffset(
                        startOffset, pyatspi.TEXT_BOUNDARY_WORD_START)
                self.assertEquals(startOffset,
                        string.find(text, startOffset, endOffset))
                startOffset = endOffset

        splitByOffsets(entry1, s1)
        splitByOffsets(entry2, s2)

    # 'combovalue' (read/write string):
    def testSetComboValue(self):
        self.runDemo('Combo boxes')
        wnd = self.app.window('Combo boxes')
        combo1 = wnd.child('Some stock icons').child(roleName = 'combo box')
        combo1.combovalue = 'Clear'
        self.assertEquals(combo1.combovalue, 'Clear')

    # 'stateSet' (read-only StateSet instance):
    def testGetStateSet(self):
        "Node.sensitive should be False for the gtk-demo app node"
        self.assert_(not self.app.sensitive)

    def testSetStateSet(self):
        "Node.stateSet should be read-only"
        # FIXME should be AttributeError?
        self.assertRaises(RuntimeError, self.app.__setattr__,  "states", pyatspi.StateSet())

    # 'relations' (read-only list of atspi.Relation instances):
    def testGetRelations(self):
        # FIXME once relations are used for something other than labels
        pass

    # 'labelee' (read-only list of Node instances):
    def testGetLabelee(self):
        "Entry1/2's labelee should be a text widget"
        self.runDemo('Dialog and Message Boxes')
        wnd = self.app.window('Dialogs')
        label = wnd.child(roleName = 'label')
        self.assertEquals(label.labelee.roleName, 'text')


    def testSetLabelee(self):
        "Node.labelee should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "labellee", None)

    # 'labeler' (read-only list of Node instances):
    def testGetLabeler(self):
        "The text areas in the 'Dialogs' window should have labelers."
        self.runDemo('Dialog and Message Boxes')
        wnd = self.app.window('Dialogs')
        text = wnd.child(roleName = 'text')
        self.assertEquals(text.labeler.name, 'Entry 2')


    def testSetLabeller(self):
        "Node.labeller should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "labeller", None)

    # 'sensitive' (read-only boolean):
    def testGetSensitive(self):
        """
        Node.sensitive should not be set for the gtk-demo app.
        It should be set for the window within the app.
        """
        self.assert_(not self.app.sensitive)
        self.assert_(self.app.children[0].sensitive)

    def testSetSensitive(self):
        "Node.sensitive should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "sensitive", True)

    # 'showing' (read-only boolean):
    def testGetShowing(self):
        "Node.showing should not be set for the gtk-demo.  It should be set for the window within the app"
        self.assert_(not self.app.showing)
        self.assert_(self.app.children[0].showing)

    def testSetShowing(self):
        "Node.showing should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "showing", True)

    # 'actions' (read-only list of Action instances):
    def testGetActions(self):
        "Node.actions should be an empty list for the app node"
        self.assertEquals(len(self.app.actions), 0)

    def testSetActions(self):
        "Node.actions should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "actions", {})

    # 'extents' (readonly tuple):
    def testGetExtents(self):
        "Node.extents should be a 4-tuple for a window, with non-zero size"
        (x,y,w,h) = self.app.children[0].extents
        self.assert_(w>0)
        self.assert_(h>0)

    def testSetExtents(self):
        "Node.extents should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "extents", (0,0,640,480))

    # 'position' (readonly tuple):
    def testGetPosition(self):
        "Node.position should be a 2-tuple for a window"
        (x,y) = self.app.children[0].position

    def testSetPosition(self):
        "Node.position should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "position", (0,0))

    # 'size' (readonly tuple):
    def testGetSize(self):
        "Node.size should be a 2-tuple for a window, with non-zero values"
        (w,h) = self.app.children[0].size
        self.assert_(w>0)
        self.assert_(h>0)

    def testSetSize(self):
        "Node.size should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "size", (640,480))

    # 'toolkitName' (readonly string):
    def testGetToolkit(self):
        self.assertEquals(self.app.toolkitName, "GAIL")

    def testSetToolkit(self):
        "Node.toolkit should be read-only"
        self.assertRaises(AttributeError, self.app.__setattr__,  "toolkitName", "GAIL")

    # 'ID'
    def testGetID(self):
        "Node.id should be numeric"
        self.assertEquals(type(self.app.id), type(42))

    def testSetID(self):
        "Node.id should be read-only"
        self.assertRaises(AttributeError, setattr, self.app, "id", 42)

class TestSelection(GtkDemoTest):
    def testTabs(self):
        """
        Tabs in the gtk-demo should be selectable, and be queryable for
        "isSelected", and the results should change as they are selected.
        """
        # Use the Info/Source tabs of gtk-demo:
        info = self.app.child('Info')
        source = self.app.child('Source')

        # Check initial state:
        self.assert_(info.isSelected)
        self.assert_(not source.isSelected)

        # Select other tab:
        source.select()

        # Check new state:
        self.assert_(not info.isSelected, False)
        self.assert_(source.isSelected)

        # Deselect tab:
        #source.deselect()

        # Check state:
        #self.assert_(info.isSelected)
        #self.assert_(not source.isSelected)

class TestValue(GtkDemoTest):
    def testGetValue(self):
        "The scrollbar starts out at position zero."
        sb = self.app.child(roleName = 'scroll bar')
        self.assertEquals(sb.value, 0)

    def testSetValue(self):
        "Ensure that we can set the value of the scrollbar."
        sb = self.app.child(roleName = 'scroll bar')
        sb.value = 100
        self.assertEquals(sb.value, 100)

    def testMinValue(self):
        "Ensure that the minimum value for the scrollbar is correct."
        sb = self.app.child(roleName = 'scroll bar')
        self.assertEquals(sb.minValue, 0)

    def testMaxValue(self):
        "Ensure that the maximum value for the scrollbar is plausible."
        sb = self.app.child(roleName = 'scroll bar')
        self.assert_(sb.maxValue > 250)

    def testMinValueIncrement(self):
        "Ensure that the minimum value increment of the scrollbar is an int."
        sb = self.app.child(roleName = 'scroll bar')
        self.assertEquals(sb.minValueIncrement, sb.minValueIncrement)


class TestSearching(GtkDemoTest):
    # FIXME: should test the various predicates and the search methods of Node
    def testFindChildren(self):
        """
        Ensure that there are the correct number of table cells in the list
        of demos.
        """
        pred = dogtail.predicate.GenericPredicate(roleName = 'table cell')
        tableCells = self.app.findChildren(pred)

        def get_table_cells_recursively(node):
            counter = 0
            for child in node.children:
                if child.roleName == 'table cell': counter += 1
                counter += get_table_cells_recursively(child)
            return counter

        counter = get_table_cells_recursively(self.app)
        self.assertEquals(len(tableCells), counter)

    def testFindChildren2(self):
        "Ensure that there are two tabs in the second page tab list."
        pred = dogtail.predicate.GenericPredicate(roleName = 'page tab list')
        pageTabLists = self.app.findChildren(pred)
        pred = dogtail.predicate.GenericPredicate(roleName = 'page tab')
        # The second page tab list is the one with the 'Info' and 'Source' tabs
        pageTabs = pageTabLists[1].findChildren(pred)
        self.assertEquals(len(pageTabs), 2)

    def testFindChildrenNonRecursive(self):
        """
        Ensure that there are the correct number of table cells in the Tree
        Store demo.
        """
        # The next several lines exist to expand the 'Tree View' item and
        # scroll down, so that runDemo() will work.
        # FIXME: make runDemo() handle this for us.
        treeViewCell = self.app.child('Tree View', roleName = 'table cell')
        treeViewCell.typeText('+')
        dogtail.tree.doDelay()
        sb = self.app.child(roleName = 'scroll bar')
        sb.value = sb.maxValue
        self.runDemo('Tree Store')
        wnd = self.app.window('Card planning sheet')
        table = wnd.child(roleName = 'tree table')
        pred = dogtail.predicate.GenericPredicate(roleName = 'table cell')
        dogtail.config.config.childrenLimit = 10000
        cells = table.findChildren(pred, recursive = False)
        direct_cells = filter(lambda cell: cell.roleName=='table cell',  table.children)
        self.assertEquals(len(cells), len(direct_cells))


class TestActions(GtkDemoTest):
    # FIXME: should test the various actions
    pass

class TestProcedural(GtkDemoTest):
    # FIXME: should test the procedural API
    pass

class TestExceptions(GtkDemoTest):
    def test_exception(self):
        # Kill the gtk-demo prematurely:
        import os, signal
        os.kill(self.pid, signal.SIGKILL)

        import gi._glib
        # Ensure that we get an exception when we try to work further with it:
        self.assertRaises(gi._glib.GError, self.app.dump)

class TestConfiguration(unittest.TestCase):
    def test_get_set_all_properties(self):
        for option in dogtail.config.config.defaults.keys():
            print("Setting config.%s property" % option)
            value = ''
            if 'Dir' in option: value = '/tmp/dogtail/'  # Special value for dir-related properties
            dogtail.config.config.__setattr__(option, value)
            self.assertEquals(dogtail.config.config.__getattr__(option), value)

    def test_default_directories_created(self):
        import os.path
        self.assertEquals(os.path.isdir(dogtail.config.config.scratchDir), True)
        self.assertEquals(os.path.isdir(dogtail.config.config.logDir), True)
        self.assertEquals(os.path.isdir(dogtail.config.config.dataDir), True)

    def test_set(self):
        self.assertRaises(AttributeError, setattr, dogtail.config.config, 'nosuchoption', 42)

    def test_get(self):
        self.assertRaises(AttributeError, getattr, dogtail.config.config, 'nosuchoption')

    def helper_create_directory_and_set_option(self, path, property_name):
        import os.path
        if os.path.isdir(path):
            import shutil
            shutil.rmtree(path)
        dogtail.config.config.__setattr__(property_name, path)
        self.assertEquals(os.path.isdir(path), True)

    def test_create_scratch_directory(self):
        new_folder = "/tmp/dt"
        self.helper_create_directory_and_set_option(new_folder, 'scratchDir')

    def test_create_data_directory(self):
        new_folder = "/tmp/dt_data"
        self.helper_create_directory_and_set_option(new_folder, 'dataDir')

    def test_create_log_directory(self):
        new_folder = "/tmp/dt_log"
        self.helper_create_directory_and_set_option(new_folder, 'logDir')

    def test_load(self):
        dogtail.config.config.load({'actionDelay': 2.0})
        self.assertEquals(dogtail.config.config.actionDelay, 2.0)

    def test_reset(self):
        default_actionDelay = dogtail.config.config.defaults['actionDelay']
        dogtail.config.config.actionDelay = 2.0
        dogtail.config.config.reset()
        self.assertEquals(dogtail.config.config.actionDelay, default_actionDelay)

def trap_stdout(function, args=None):
    import sys
    from StringIO import StringIO

    saved_stdout = sys.stdout
    try:
        out = StringIO()
        sys.stdout = out
        import types
        if type(args) is dict:
            function(**args)
        elif args:
            function(args)
        else:
            function()
        output = out.getvalue().strip()
    finally:
        sys.stdout = saved_stdout
    return output

class TestDump(GtkDemoTest):

    def test_dump_to_stdout(self):
        child = self.app.child('Source')
        output = trap_stdout(child.dump)
        self.assertEquals(output,
            """[page tab | Source]
 [scroll pane | ]
  [text | ]
  [scroll bar | ]
   [action | activate |  ]
  [scroll bar | ]
   [action | activate |  ]""")

if __name__ == '__main__':
    unittest.main()



"""Predicates that can be used when searching for nodes.

Author: David Malcolm <dmalcolm@redhat.com>"""
__author__ = 'David Malcolm <dmalcolm@redhat.com>'

import unittest

from i18n import TranslatableString

def stringMatches(scriptName, reportedName):
    assert isinstance(scriptName, TranslatableString)

    return scriptName.matchedBy(reportedName)

def makeScriptRecursiveArgument(isRecursive, defaultValue):
    if isRecursive==defaultValue:
        return ""
    else:
        return ", recursive=%s"%isRecursive

def makeCamel(string):
    """
    Convert string to camelCaps
    """
    string = str(string)
    # FIXME: this function is probably really fragile, lots of difficult cases here

    # Sanitize string, replacing bad characters with spaces:
    for char in ":;!@#$%^&*()-+=_~`\\/?|[]{}<>,.\t\n\r\"'":
        string = string.replace(char, " ")
    words = string.strip().split(" ")
    for word in words:
        word.strip
    result = ""
    firstWord=True
    for word in words:
        lowercaseWord = word.lower()
        if firstWord:
            result += lowercaseWord
            firstWord=False
        else:
            result += lowercaseWord.capitalize()
    return result

class Predicate:
    """Abstract base class representing a predicate function on nodes.

    It's more than just a function in that it has data and can describe itself"""
    def satisfiedByNode(self, node):
        """Pure virtual method returning a boolean if the predicate is satisfied by the node"""
        raise NotImplementedError

    def describeSearchResult(self, node):
        raise NotImplementedError

    def makeScriptMethodCall(self, isRecursive):
        """
        Method to generate a string containing a (hopefully) readable search
        method call on a node (to be used when generating Python source code in
        the event recorder)
        """
        raise NotImplementedError

    def makeScriptVariableName(self):
        """
        Method to generate a string containing a (hopefully) readable name
        for a Node instance variable that would be the result of a search on
        this predicate (to be used when generating Python source code in the
        event recorder).
        """
        raise NotImplementedError

    def __eq__(self, other):
        """
        Predicates are considered equal if they are of the same subclass and
        have the same data
        """
        # print "predeq: self:%s"%self
        # print "               other:%s"%other
        # print "predeq: selfdict:%s"%self.__dict__
        # print "               otherdict:%s"%other.__dict__

        if type(self)!=type(other):
            return False
        else:
            return self.__dict__ == other.__dict__


class IsAnApplicationNamed(Predicate):
    """Search subclass that looks for an application by name"""
    def __init__(self, appName):
        self.appName = TranslatableString(appName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            return node.roleName=='application' and stringMatches(self.appName, node.name)
        return satisfiedByNode

    def describeSearchResult(self):
        return '%s application'%self.appName

    def makeScriptMethodCall(self, isRecursive):
        # ignores the isRecursive parameter
        return "application(%s)"%self.appName

    def makeScriptVariableName(self):
        return makeCamel(self.appName)+"App"

class GenericPredicate(Predicate):
    """SubtreePredicate subclass that takes various optional search fields"""

    def __init__(self, name = None, roleName = None, description= None, label = None, debugName=None):
        if name:
            self.name = TranslatableString(name)
        else:
            self.name = None
        self.roleName = roleName
        self.description = description
        if label:
            self.label = TranslatableString(label)
        else:
            self.label = None

        if debugName:
            self.debugName = debugName
        else:
            if label:
                self.debugName = "labelled '%s'"%self.label
            else:
                self.debugName = "child with"
            if name:
                self.debugName += " name=%s" % self.name
            if roleName:
                self.debugName += " roleName='%s'"%roleName
            if description:
                self.debugName += " description='%s'"%description
        assert self.debugName

        self.satisfiedByNode = self._genCompareFunc()


    def _genCompareFunc(self):
        def satisfiedByNode(node):
            # labelled nodes are handled specially:
            if self.label:
                # this reverses the search; we're looking for a node with LABELLED_BY
                # and then checking the label, rather than looking for a label and
                # then returning whatever LABEL_FOR targets
                if node.labeller:
                    return stringMatches(self.label, node.labeller.name)
                else: return False
            else:
                # Ensure the node matches any criteria that were set:
                if self.name:
                    if not stringMatches(self.name,node.name): return False
                if self.roleName:
                    if self.roleName!=node.roleName: return False
                if self.description:
                    if self.description!=node.description: return False
                return True
        return satisfiedByNode

    def describeSearchResult(self):
        return self.debugName

    def makeScriptMethodCall(self, isRecursive):
        if self.label:
            args = "label='%s'"%label
        else:
            args = ""
            if self.name:
                print self.name
                args += " name=%s"%self.name
            if self.roleName:
                args += " roleName='%s'"%self.roleName
            if self.description:
                args += " description='%s'"%self.description
        return "child(%s%s)"%(args, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        if self.label:
            return makeCamel(self.label)+"Node"
        else:
            if self.name:
                return makeCamel(self.name)+"Node"
            if self.roleName:
                return makeCamel(self.roleName)+"Node"
            if self.description:
                return makeCamel(self.description)+"Node"

class IsNamed(Predicate):
    """Predicate subclass that looks simply by name"""

    def __init__(self, name):
        self.name = TranslatableString(name)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            return stringMatches(self.name, node.name)
        return satisfiedByNode

    def describeSearchResult(self):
        return "named %s"%self.name

    def makeScriptMethodCall(self, isRecursive):
        return "child(name=%s%s)"%(self.name, makeScriptRecursiveArgument(isRecursive, True))
    def makeScriptVariableName(self):
        return makeCamel(self.name)+"Node"

class IsAWindowNamed(Predicate):
    """Predicate subclass that looks for a top-level window by name"""
    def __init__(self, windowName):
        self.windowName = TranslatableString(windowName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            return node.roleName=='frame' and stringMatches(self.windowName, node.name)
        return satisfiedByNode

    def describeSearchResult(self):
        return "%s window"%self.windowName

    def makeScriptMethodCall(self, isRecursive):
        return "window(%s%s)"%(self.windowName, makeScriptRecursiveArgument(isRecursive, False))

    def makeScriptVariableName(self):
        return makeCamel(self.windowName)+"Win"

class IsAWindow(Predicate):
    """Predicate subclass that looks for top-level windows"""
    def __init__(self):
        self.satisfiedByNode = lambda node: node.roleName == 'frame'

    def describeSearchResult(self):
        return "window"

class IsADialogNamed(Predicate):
    """Predicate subclass that looks for a top-level dialog by name"""
    def __init__(self, dialogName):
        self.dialogName = TranslatableString(dialogName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            return node.roleName=='dialog' and stringMatches(self.dialogName, node.name)
        return satisfiedByNode

    def describeSearchResult(self):
        return '%s dialog'%self.dialogName

    def makeScriptMethodCall(self, isRecursive):
        return "dialog(%s%s)"%(self.dialogName, makeScriptRecursiveArgument(isRecursive, False))

    def makeScriptVariableName(self):
        return makeCamel(self.dialogName)+"Dlg"

class IsLabelledBy(Predicate):
    """Predicate: is this node labelled by another node"""
    pass

class IsLabelledAs(Predicate):
    """Predicate: is this node labelled with the text string (i.e. by another node with that as a name)"""
    def __init__(self, labelText):
        self.labelText = TranslatableString(labelText)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            # FIXME
            if node.labeller:
                return stringMatches(self.labelText, node.labeller.name)
            else: return False
        return satisfiedByNode

    def describeSearchResult(self):
        return 'labelled %s'%self.labelText

    def makeScriptMethodCall(self, isRecursive):
        return "child(label=%s%s)"%(self.labelText, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.labelText)+"Node"

class IsAMenuNamed(Predicate):
    """Predicate subclass that looks for a menu by name"""
    def __init__(self, menuName):
        self.menuName = TranslatableString(menuName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName=='menu' and \
                stringMatches(self.menuName, node.name)

    def describeSearchResult(self):
        return '%s menu'%(self.menuName)

    def makeScriptMethodCall(self, isRecursive):
        return "menu(%s%s)"%(self.menuName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.menuName)+"Menu"

class IsAMenuItemNamed(Predicate):
    """Predicate subclass that looks for a menu item by name"""
    def __init__(self, menuItemName):
        self.menuItemName = TranslatableString(menuItemName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: \
                node.roleName.endswith('menu item') and \
                stringMatches(self.menuItemName, node.name)

    def describeSearchResult(self):
        return '%s menuitem'%(self.menuItemName)

    def makeScriptMethodCall(self, isRecursive):
        return "menuItem(%s%s)"%(self.menuItemName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.menuItemName)+"MenuItem"

class IsATextEntryNamed(Predicate):
    """Predicate subclass that looks for a text entry by name"""
    def __init__(self, textEntryName):
        self.textEntryName = TranslatableString(textEntryName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName == 'text' and \
                stringMatches(self.textEntryName, node.name)

    def describeSearchResult(self):
        return '%s textentry'%(self.textEntryName)

    def makeScriptMethodCall(self, isRecursive):
        return "textentry(%s%s)"%(self.textEntryName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.textEntryName)+"Entry"

class IsAButtonNamed(Predicate):
    """Predicate subclass that looks for a button by name"""
    def __init__(self, buttonName):
        self.buttonName = TranslatableString(buttonName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName == 'push button' \
                and stringMatches(self.buttonName, node.name)

    def describeSearchResult(self):
        return '%s button'%(self.buttonName)

    def makeScriptMethodCall(self, isRecursive):
        return "button(%s%s)"%(self.buttonName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.buttonName)+"Button"

class IsATabNamed(Predicate):
    """Predicate subclass that looks for a tab by name"""
    def __init__(self, tabName):
        self.tabName = TranslatableString(tabName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName=='page tab' and \
                stringMatches(self.tabName, node.name)

    def describeSearchResult(self):
        return '%s tab'%(self.tabName)

    def makeScriptMethodCall(self, isRecursive):
        return "tab(%s%s)"%(self.tabName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.tabName)+"Tab"

class PredicateTests(unittest.TestCase):
    def testCapitalization(self):
        self.assertEquals(makeCamel("gnome-terminal"),"gnomeTerminal")
        self.assertEquals(makeCamel("Evolution - Mail"), "evolutionMail")
        self.assertEquals(makeCamel('self.assertEquals(makeCamel("Evolution - Mail"), "evolutionMail")'), "selfAssertequalsMakecamelEvolutionMailEvolutionmail")

    def testPredicates(self):
        class DummyNode:
            def __init__(self, name = '', roleName = '', description = ''):
                self.name = name
                self.roleName = roleName
                self.description = description
                self.labeller = None

        dn1 = DummyNode('dummy name 1', 'dummy role 1', 'dummy desc 1')
        dn2 = DummyNode('dummy name 2', 'dummy role 2', 'dummy desc 2')

        dummyApp = DummyNode('dummy', 'application')
        self.assertTrue(IsAnApplicationNamed(dummyApp.name).satisfiedByNode(dummyApp))

        self.assertTrue(IsNamed(dn1.name).satisfiedByNode(dn1))

        dummyWin = DummyNode('dummy', 'frame')
        self.assertTrue(IsAWindowNamed(dummyWin.name).satisfiedByNode(dummyWin))
        self.assertTrue(IsAWindow().satisfiedByNode(dummyWin))

        dummyDlg = DummyNode('dummy', 'dialog')
        self.assertTrue(IsADialogNamed(dummyDlg.name).satisfiedByNode(dummyDlg))

        dn2.labeller = dn1
        #IsLabelledBy
        self.assertTrue(IsLabelledAs(dn1.name).satisfiedByNode(dn2))

        dummyMenu = DummyNode('dummy', 'menu')
        self.assertTrue(IsAMenuNamed(dummyMenu.name).satisfiedByNode(dummyMenu))

        dummyMenuItem = DummyNode('dummy', 'menu item')
        self.assertTrue(IsAMenuItemNamed(dummyMenuItem.name).satisfiedByNode(dummyMenuItem))

        dummyText = DummyNode('dummy', 'text')
        self.assertTrue(IsATextEntryNamed(dummyText.name).satisfiedByNode(dummyText))

        dummyButton = DummyNode('dummy', 'push button')
        self.assertTrue(IsAButtonNamed(dummyButton.name).satisfiedByNode(dummyButton))

        dummyTab = DummyNode('dummy', 'page tab')
        self.assertTrue(IsATabNamed(dummyTab.name).satisfiedByNode(dummyTab))

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
"""
Unit tests for the dogtail.procedural API
"""
__author__ = "Zack Cerza <zcerza@redhat.com>"

import unittest
from dogtail.procedural import *
config.logDebugToFile = False
config.logDebugToStdOut = True
import pyatspi
import Node


class GtkDemoTest(Node.GtkDemoTest):
    def setUp(self):
        self.pid = run('gtk-demo')
        self.app = focus.application.node

    #FIXME: Implement doubleclick() in d.procedural and override the other
    # methods of Node.GtkDemoTest


class TestFocusApplication(GtkDemoTest):
    def testFocusingBogusNameWithoutAFatalError(self):
        config.fatalErrors = False
        output = Node.trap_stdout(focus.application, "should not be found")
        self.assertTrue('The requested widget could not be focused: "should not be found" application' in output)

    def testThrowExceptionOnFocusingBogusName(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.application, "should not be found")

    def testFocusingBasic(self):
        "Ensure that focus.application() sets focus.application.node properly"
        focus.application.node = None
        focus.application("gtk-demo")
        self.assertEquals(focus.application.node, self.app)


class TestFocusWindow(GtkDemoTest):
    def testFocusingBogusNameWithoutAFatalError(self):
        config.fatalErrors = False
        output = Node.trap_stdout(focus.window, "should not be found")
        self.assertEquals(focus.window.node, None)
        self.assertTrue('The requested widget could not be focused: "should not be found" window' in output)

    def testThrowExceptionOnFocusingBogusName(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.window, "should not be found")


class TestFocusDialog(GtkDemoTest):
    def testFocusingBogusNameWithoutAFatalError(self):
        config.fatalErrors = False
        output = Node.trap_stdout(focus.dialog, "should not be found")
        self.assertEquals(focus.dialog.node, None)
        self.assertTrue('The requested widget could not be focused: "should not be found" dialog' in output)

    def testThrowExceptionOnFocusingBogusName(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.dialog, "should not be found")


class TestFocusWidget(GtkDemoTest):
    def testFocusingEmptyName(self):
        self.assertRaises(TypeError, focus.widget)

    def testFocusingBogusNameWithoutAFatalError(self):
        config.fatalErrors = False
        output = Node.trap_stdout(focus.widget, "should not be found")
        self.assertEquals(focus.widget.node, None)
        self.assertTrue('The requested widget could not be focused: child with name="should not be found"' in output)

    def testThrowExceptionOnFocusingBogusName(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.widget, "should not be found")

    def testFocusingBasic(self):
        "Ensure that focus.widget('foo') finds a node with name 'foo'"
        focus.widget("Application main window")
        self.assertEquals(focus.widget.name, "Application main window")


class TestFocus(GtkDemoTest):
    def testInitialState(self):
        "Ensure that focus.widget, focus.dialog and focus.window are None " + \
                "initially."
        self.assertEquals(focus.widget.node, None)
        self.assertEquals(focus.dialog.node, None)
        self.assertEquals(focus.window.node, None)

    def testFocusingApp(self):
        "Ensure that focus.app() works"
        focus.app.node = None
        focus.app('gtk-demo')
        self.assertEquals(focus.app.node, self.app)

    def testFocusingAppViaApplication(self):
        "Ensure that focus.application() works"
        focus.app.node = None
        focus.application('gtk-demo')
        self.assertEquals(focus.app.node, self.app)

    def testFocusGettingBogusAttribute(self):
        self.assertRaises(AttributeError, getattr, focus, 'nosuchtype')

    def testFocusSettingBogusAttribute(self):
        self.assertRaises(AttributeError, setattr, focus, 'nosuchtype', 'nothing')

    def testFocusingRoleName(self):
        "Ensure that focus.widget(roleName=...) works."
        focus.widget(roleName='page tab')
        self.assert_(isinstance(focus.widget.node, tree.Node))
        self.assertEquals(focus.widget.node.role, pyatspi.ROLE_PAGE_TAB)

    def testFocusMenu(self):
        self.runDemo('Application main window')
        focus.window('Application Window')
        focus.menu('File')
        self.assert_(isinstance(focus.widget.node, tree.Node))
        self.assertEquals(focus.widget.node.role, pyatspi.ROLE_MENU)

    def testFocusMenuItem(self):
        self.runDemo('Application main window')
        focus.window('Application Window')
        click.menu('File')
        focus.menuItem('New')
        self.assert_(isinstance(focus.widget.node, tree.Node))
        self.assertEquals(focus.widget.node.role, pyatspi.ROLE_MENU_ITEM)

    def testFocusButton(self):
        self.runDemo('Application main window')
        focus.window('Application Window')
        focus.button('Open')
        self.assert_(isinstance(focus.widget.node, tree.Node))
        self.assertEquals(focus.widget.node.role, pyatspi.ROLE_PUSH_BUTTON)

    def testFocusTable(self):
        self.runDemo('Builder')
        focus.window('GtkBuilder demo')
        focus.table('')
        self.assert_(isinstance(focus.widget.node, tree.Node))
        self.assertEquals(focus.widget.node.role, pyatspi.ROLE_TABLE)

    def testFocusTableCell(self):
        self.runDemo('Builder')
        focus.window('GtkBuilder demo')
        focus.tableCell('')
        self.assert_(isinstance(focus.widget.node, tree.Node))
        self.assertEquals(focus.widget.node.role, pyatspi.ROLE_TABLE_CELL)

    def testFocusText(self):
        self.runDemo('Application main window')
        focus.window('Application Window')
        focus.text('')
        self.assert_(isinstance(focus.widget.node, tree.Node))
        self.assertEquals(focus.widget.node.role, pyatspi.ROLE_TEXT)


class TestKeyCombo(GtkDemoTest):
    def testKeyCombo(self):
        self.runDemo('Application main window')
        focus.window('Application Window')
        keyCombo("<ctrl>a")
        focus.dialog('About GTK+ Code Demos')


class TestActions(GtkDemoTest):
    def testClick(self):
        click('Source')
        self.assertTrue(focus.widget.isSelected)

    def testClickWithRaw(self):
        click('Source', raw=True)
        self.assertTrue(focus.widget.isSelected)

    def testSelect(self):
        select('Source')
        self.assertTrue(focus.widget.isSelected)

    def testDeselect(self):
        type('Icon View')
        click('Icon View')
        type('+')
        self.runDemo('Icon View Basics')
        focus.window('GtkIconView demo')

        focus.widget(roleName='icon')
        select()
        deselect()
        self.assertFalse(focus.widget.isSelected)

    def testTyping(self):
        self.runDemo('Dialog and Message Boxes')
        focus.window('Dialogs')
        focus.widget(roleName='text')
        type("hello world")
        self.assertEquals(focus.widget.node.text, 'hello world')


if __name__ == "__main__":
    unittest.main()



"""
Dogtail's procedural UI
All the classes here are intended to be single-instance, except for Action.
"""
__author__ = 'Zack Cerza <zcerza@redhat.com>'
##############################################################################
#                                                                            #
# WARNING: Here There Be Dragons (TM)                                        #
#                                                                            #
# If you don't understand how to use this API, you almost certainly don't    #
# want to read the code first. We make use of some very non-intuitive        #
# features of Python in order to make the API very simplistic. Therefore,    #
# you should probably only read this code if you're already familiar with    #
# some of Python's advanced features. You have been warned. ;)               #
#                                                                            #
##############################################################################

import tree
import predicate
from config import config
from utils import Lock
import rawinput

#FocusError = "FocusError: %s not found"
class FocusError(Exception):
    pass

import errors
def focusFailed(pred):
    errors.warn('The requested widget could not be focused: %s' % pred.debugName)

ENOARGS = "At least one argument is needed"

class FocusBase:
    """
    The base for every class in the module. Does nothing special, really.
    """
    node = None

    def __getattr__ (self, name):
        # Fold all the Node's AT-SPI properties into the Focus object.
        try: return getattr(self.node, name)
        except AttributeError:
            raise AttributeError, name

    def __setattr__ (self, name, value):
        # Fold all the Node's AT-SPI properties into the Focus object.
        if name == 'node':
            self.__class__.__dict__[name] = value
        else:
            try: setattr(self.node, name, value)
            except AttributeError:
                raise AttributeError, name

class FocusApplication (FocusBase):
    """
    Keeps track of which application is currently focused.
    """
    desktop = tree.root
    def __call__ (self, name):
        """
        Search for an application that matches and refocus on the given name.
        """
        try:
            pred = predicate.IsAnApplicationNamed(name)
            app = self.desktop.findChild(pred, recursive = False, retry = False)
        except tree.SearchError, desc:
            if config.fatalErrors: raise FocusError, name
            else:
                focusFailed(pred)
                return False
        if app:
            FocusApplication.node = app
            FocusDialog.node = None
            FocusWindow.node = None
            FocusWidget.node = None
        return True

class FocusDesktop (FocusBase):
    """
    This isn't used yet, and may never be used.
    """
    pass

class FocusWindow (FocusBase):
    """
    Keeps track of which window is currently focused.
    """
    def __call__ (self, name):
        """
        Search for a dialog that matches the given name and refocus on it.
        """
        result = None
        pred = predicate.IsAWindowNamed(name)
        try:
            result = FocusApplication.node.findChild(pred, requireResult=False, recursive = False)
        except AttributeError: pass
        if result:
            FocusWindow.node = result
            FocusDialog.node = None
            FocusWidget.node = None
        else:
            if config.fatalErrors: raise FocusError, pred.debugName
            else:
                focusFailed(pred)
                return False
        return True

class FocusDialog (FocusBase):
    """
    Keeps track of which dialog is currently focused.
    """
    def __call__ (self, name):
        """
        Search for a dialog that matches the given name and refocus on it.
        """
        result = None
        pred = predicate.IsADialogNamed(name)
        try:
            result = FocusApplication.node.findChild(pred, requireResult=False, recursive = False)
        except AttributeError: pass
        if result:
            FocusDialog.node = result
            FocusWidget.node = None
        else:
            if config.fatalErrors: raise FocusError, pred.debugName
            else:
                focusFailed(pred)
                return False
        return True

class FocusWidget (FocusBase):
    """
    Keeps track of which widget is currently focused.
    """
    def findByPredicate(self, pred):
        result = None
        try:
            result = FocusWidget.node.findChild(pred, requireResult = False, retry = False)
        except AttributeError: pass
        if result: FocusWidget.node = result
        else:
            try:
                result = FocusDialog.node.findChild(pred, requireResult = False, retry = False)
            except AttributeError: pass
        if result: FocusWidget.node = result
        else:
            try:
                result = FocusWindow.node.findChild(pred, requireResult = False, retry = False)
            except AttributeError: pass
        if result: FocusWidget.node = result
        else:
            try:
                result = FocusApplication.node.findChild(pred, requireResult = False, retry = False)
                if result: FocusWidget.node = result
            except AttributeError:
                if config.fatalErrors: raise FocusError, name
                else:
                    focusFailed(pred)
                    return False

        if result == None:
            FocusWidget.node = result
            if config.fatalErrors: raise FocusError, pred.debugName
            else:
                focusFailed(pred)
                return False
        return True

    def __call__ (self, name = '', roleName = '', description = ''):
        """
        If name, roleName or description are specified, search for a widget that matches and refocus on it.
        """
        if not name and not roleName and not description:
            raise TypeError, ENOARGS

        # search for a widget.
        pred = predicate.GenericPredicate(name = name,
                roleName = roleName, description = description)
        return self.findByPredicate(pred)

class Focus (FocusBase):
    """
    The container class for the focused application, dialog and widget.
    """

    def __getattr__ (self, name):
        raise AttributeError, name
    def __setattr__(self, name, value):
        if name in ('application', 'dialog', 'widget', 'window'):
            self.__dict__[name] = value
        else:
            raise AttributeError, name

    desktop = tree.root
    application = FocusApplication()
    app = application # shortcut :)
    dialog = FocusDialog()
    window = FocusWindow()
    frame = window
    widget = FocusWidget()

    def button (self, name):
        """
        A shortcut to self.widget.findByPredicate(predicate.IsAButtonNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsAButtonNamed(name))

    def icon (self, name):
        """
        A shortcut to self.widget(name, roleName = 'icon')
        """
        return self.widget(name = name, roleName = 'icon')

    def menu (self, name):
        """
        A shortcut to self.widget.findByPredicate(predicate.IsAMenuNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsAMenuNamed(name))

    def menuItem (self, name):
        """
        A shortcut to self.widget.findByPredicate(predicate.IsAMenuItemNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsAMenuItemNamed(name))

    def table (self, name = ''):
        """
        A shortcut to self.widget(name, roleName 'table')
        """
        return self.widget(name = name, roleName = 'table')

    def tableCell (self, name = ''):
        """
        A shortcut to self.widget(name, roleName 'table cell')
        """
        return self.widget(name = name, roleName = 'table cell')

    def text (self, name = ''):
        """
        A shortcut to self.widget.findByPredicate(IsATextEntryNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsATextEntryNamed(name))

class Action (FocusWidget):
    """
    Aids in executing AT-SPI actions, refocusing the widget if necessary.
    """
    def __init__ (self, action):
        """
        action is a string with the same name as the AT-SPI action you wish to execute using this class.
        """
        self.action = action

    def __call__ (self, name = '', roleName = '', description = '', delay = config.actionDelay):
        """
        If name, roleName or description are specified, first search for a widget that matches and refocus on it.
        Then execute the action.
        """
        if name or roleName or description:
            FocusWidget.__call__(self, name = name, roleName = roleName, description = description)
        self.node.doActionNamed(self.action)

    def __getattr__ (self, attr):
        return getattr(FocusWidget.node, attr)

    def __setattr__ (self, attr, value):
        if attr == 'action':
            self.__dict__[attr] = value
        else: setattr(FocusWidget, attr, value)

    def button (self, name):
        """
        A shortcut to self(name, roleName = 'push button')
        """
        self.__call__(name = name, roleName = 'push button')

    def menu (self, name):
        """
        A shortcut to self(name, roleName = 'menu')
        """
        self.__call__(name = name, roleName = 'menu')

    def menuItem (self, name):
        """
        A shortcut to self(name, roleName = 'menu item')
        """
        self.__call__(name = name, roleName = 'menu item')

    def table (self, name = ''):
        """
        A shortcut to self(name, roleName 'table')
        """
        self.__call__(name = name, roleName = 'table')

    def tableCell (self, name = ''):
        """
        A shortcut to self(name, roleName 'table cell')
        """
        self.__call__(name = name, roleName = 'table cell')

    def text (self, name = ''):
        """
        A shortcut to self(name, roleName = 'text')
        """
        self.__call__(name = name, roleName = 'text')

class Click (Action):
    """
    A special case of Action, Click will eventually handle raw mouse events.
    """
    primary = 1
    middle = 2
    secondary = 3
    def __init__ (self):
        Action.__init__(self, 'click')

    def __call__ (self, name = '', roleName = '', description = '', raw = True, button = primary, delay = config.actionDelay):
        """
        By default, execute a raw mouse event.
        If raw is False or if button evaluates to False, just pass the rest of
        the arguments to Action.
        """
        if name or roleName or description:
            FocusWidget.__call__(self, name = name, roleName = roleName, description = description)
        if raw and button:
            # We're doing a raw mouse click
            Click.node.click(button)
        else:
            Action.__call__(self, name = name, roleName = roleName, description = description, delay = delay)

class Select (Action):
    """
    Aids in selecting and deselecting widgets, i.e. page tabs
    """
    select = 'select'
    deselect = 'deselect'
    def __init__(self, action):
        """
        action must be 'select' or 'deselect'.
        """
        if action not in (self.select, self.deselect):
            raise ValueError, action
        Action.__init__(self, action)

    def __call__ (self, name = '', roleName = '', description = '', delay = config.actionDelay):
        """
        If name, roleName or description are specified, first search for a widget that matches and refocus on it.
        Then execute the action.
        """
        if name or roleName or description:
            FocusWidget.__call__(self, name = name, roleName = roleName, description = description)
        func = getattr(self.node, self.action)
        func()

def type(text):
    if focus.widget.node:
        focus.widget.node.typeText(text)
    else:
        rawinput.typeText(text)

def keyCombo(combo):
    if focus.widget.node:
        focus.widget.node.keyCombo(combo)
    else:
        rawinput.keyCombo(combo)

def run(application, arguments = '', appName = ''):
    from utils import run as utilsRun
    pid = utilsRun(application + ' ' + arguments, appName = appName)
    focus.application(application)
    return pid

# tell sniff not to use auto-refresh while script using this module is running
sniff_lock = Lock(lockname='sniff_refresh.lock',randomize=False)
try:
    sniff_lock.lock()
except OSError:
    pass # lock was already present from other script instance or leftover from killed instance
# lock should unlock automatically on script exit.

focus = Focus()
click = Click()
activate = Action('activate')
openItem = Action('open')
menu = Action('menu')
select = Select(Select.select)
deselect = Select(Select.deselect)



# -*- coding: utf-8 -*-
"""
Logging facilities

Authors: Ed Rousseau <rousseau@redhat.com>, Zack Cerza <zcerza@redhat.com, David Malcolm <dmalcolm@redhat.com>
"""

__author__ = """Ed Rousseau <rousseau@redhat.com>,
Zack Cerza <zcerza@redhat.com,
David Malcolm <dmalcolm@redhat.com>
"""
import os
import sys
import time
import datetime
from config import config
import codecs

# Timestamp class for file logs
class TimeStamp:
    """
    Generates timestamps tempfiles and log entries
    """
    def __init__(self):
        self.now = "0"
        self.timetup = time.localtime()

    def zeroPad(self, int, width = 2):
        """
        Pads an integer 'int' with zeroes, up to width 'width'.

        Returns a string.

        It will not truncate. If you call zeroPad(100, 2), '100' will be returned.
        """
        if int < 10 ** width:
            return ("0" * (width - len(str(int))) ) + str(int)
        else:
            return str(int)

    # file stamper
    def fileStamp(self, filename, addTime = True):
        """
        Generates a filename stamp in the format of filename_YYYYMMDD-hhmmss.
        A format of filename_YYYYMMDD can be used instead by specifying addTime = False.
        """
        self.now = filename.strip() + "_"
        self.timetup = time.localtime()

        # Should produce rel-eng style filestamps
        # format it all pretty by chopping the tuple
        fieldCount = 3
        if addTime: fieldCount = fieldCount + 3
        for i in range(fieldCount):
            if i == 3: self.now = self.now + '-'
            self.now = self.now + self.zeroPad(self.timetup[i])
        return self.now

    # Log entry stamper
    def entryStamp(self):
        """
        Generates a logfile entry stamp of YYYY.MM.DD HH:MM:SS
        """
        self.timetup = time.localtime()

        # This will return a log entry formatted string in YYYY.MM.DD HH:MM:SS
        for i in range(6):
            # put in the year
            if i == 0:
                self.now = str(self.timetup[i])
            # Format Month and Day
            elif i == 1 or i == 2:
                self.now = self.now + "." + self.zeroPad(self.timetup[i])
            else:
                x = self.timetup[i]
                # make the " " between Day and Hour and put in the hour
                if i == 3:
                    self.now = self.now + " " + self.zeroPad(self.timetup[i])
                # Otherwise Use the ":" divider
                else:
                    self.now = self.now + ":" + self.zeroPad(self.timetup[i])
        return self.now

class Logger:
    """
    Writes entries to standard out.
    """
    stamper = TimeStamp()
    def __init__(self, logName, file = False, stdOut = True):
        """
        name: the name of the log
        file: The file object to log to.
        stdOut: Whether to log to standard out.
        """
        self.logName = logName
        self.stdOut = stdOut
        self.file = file # Handle to the logfile
        if not self.file: return

        scriptName = config.scriptName
        if not scriptName: scriptName = 'log'
        self.fileName = scriptName

        # check to see if we can write to the logDir
        if os.path.isdir(config.logDir):
            self.findUniqueName()
        else:
            # If path doesn't exist, raise an exception
            raise IOError, \
                    "Log path %s does not exist or is not a directory" % config.logDir

    def findUniqueName(self):
        # generate a logfile name and check if it already exists
        self.fileName = config.logDir + self.stamper.fileStamp(self.fileName) \
                + '_' + self.logName
        i = 0
        while os.path.exists(self.fileName):
            # Append the pathname
            if i == 0:
                self.fileName = self.fileName + "." + str(i)
            else:
                logsplit = self.fileName.split(".")
                logsplit[-1] = str(i)
                self.fileName = ".".join(logsplit)
            i += 1

    def createFile(self):
        # Try to create the file and write the header info
        print "Creating logfile at %s ..." % self.fileName
        self.file = codecs.open(self.fileName, mode = 'wb', encoding = \
                'utf-8')
        self.file.write("##### " + os.path.basename(self.fileName) + '\n')
        self.file.flush()

    def log(self, message, newline = True, force = False):
        """
        Hook used for logging messages. Might eventually be a virtual
        function, but nice and simple for now.

        If force is True, log to a file irrespective of config.logDebugToFile.
        """
        message = message.decode('utf-8', 'replace')

        # Try to open and write the result to the log file.
        if isinstance(self.file, bool) and (force or config.logDebugToFile):
            self.createFile()

        if force or config.logDebugToFile:
            if newline: self.file.write(message + '\n')
            else: self.file.write(message + ' ')
            self.file.flush()

        if self.stdOut and config.logDebugToStdOut:
            if newline: print message
            else: print message,

class ResultsLogger(Logger):
    """
    Writes entries into the Dogtail log
    """
    def __init__(self, stdOut = True):
        Logger.__init__(self, 'results', file = True, stdOut = stdOut)

    # Writes the result of a test case comparison to the log
    def log(self, entry):
        """
        Writes the log entry. Requires a 1 {key: value} pair dict for an argument or else it will throw an exception.
        """
        # We require a 1 key: value dict
        # Strip all leading and trailing witespace from entry dict and convert
    # to string for writing

        if len(entry) == 1:
            key = entry.keys()
            value = entry.values()
            key = key[0]
            value = value[0]
            entry = str(key) + ":      " + str(value)
        else:
            raise ValueError, entry
            print "Method argument requires a 1 {key: value} dict. Supplied argument not one {key: value}"

        Logger.log(self, self.stamper.entryStamp() + "      " + entry, force = True)

debugLogger = Logger('debug', config.logDebugToFile)

import traceback
def exceptionHook(exc, value, tb):
    tbStringList = traceback.format_exception(exc, value, tb)
    tbString = ''.join(tbStringList)
    debugLogger.log(tbString)
    sys.exc_clear()

sys.excepthook = exceptionHook



# -*- coding: utf-8 -*-
"""Test Case magic

Author: Ed Rousseau <rousseau@redhat.com>"""
__author__ = "Ed Rousseau <rousseau@redhat.com>"

import string
import sys
import os
import re
import time
import datetime
import os.path
from config import config
from logging import ResultsLogger, TimeStamp, debugLogger
from PIL import Image, ImageChops, ImageStat


class TC(object):
    """
    The Test Case Superclass
    """
    logger = ResultsLogger()
    def __init__(self):
        self.encoding = config.encoding
        # ascii + unicode. 8 bit extended char has been ripped out
        self.supportedtypes = ("ascii", "utf-8", "utf-16", "utf-16-be", "utf-16-le", "unicode-escape", "raw-unicode-escape",
        "big5", "gb18030", "eucJP", "eucKR", "shiftJIS")

    # String comparison function
    def compare(self, label, baseline, undertest, encoding=config.encoding):
        """
        Compares 2 strings to see if they are the same. The user may specify
        the encoding to which the two strings are to be normalized for the
        comparison. Default encoding is the default system encoding.
        Normalization to extended 8 bit charactersets is not supported.

        When the origin of either baseline or undertest is a text file whose
        encoding is something other than ASCII, it is necessary to use
        codecs.open() instead of open(), so the file's encoding may be
        specified.
        """
        self.label = label.strip()
        self.baseline = baseline
        self.undertest = undertest
        for string in [self.baseline, self.undertest]:
            try: string = unicode(string, 'utf-8')
            except TypeError: pass
        self.encoding = encoding

        # Normalize the encoding type for the comparaison based on self.encoding
        if self.encoding in self.supportedtypes:
            self.baseline = (self.baseline).encode(self.encoding)
            self.undertest = (self.undertest).encode(self.encoding)
            # Compare the strings
            if self.baseline == self.undertest:
                self.result = {self.label: "Passed"}
            else:
                self.result = {self.label: "Failed - " + self.encoding + " strings do not match. " + self.baseline + " expected: Got " + self.undertest}
            # Pass the test result to the ResultsLogger for writing
            TC.logger.log(self.result)
            return self.result

        else:
            # We should probably raise an exception here
            self.result = {self.label: "ERROR - " + self.encoding + " is not a supported encoding type"}
            TC.logger.log(self.result)
            return self.result


# String Test Case subclass
class TCString(TC):
    """
    String Test Case Class
    """
    def __init__(self):
        TC.__init__(self)

# Image test case subclass
class TCImage(TC):
    """
    Image Test Case Class.
    """
    def compare(self, label, baseline, undertest):
        for _file in (baseline, undertest):
            if type(_file) is not unicode and type(_file) is not str:
                raise TypeError("Need filenames!")
        self.label = label.strip()
        self.baseline = baseline.strip()
        self.undertest = undertest.strip()
        diffName = TimeStamp().fileStamp("diff") + ".png"
        self.diff = os.path.normpath(
                os.path.sep.join((config.scratchDir, diffName)))

        self.baseImage = Image.open(self.baseline)
        self.testImage = Image.open(self.undertest)
        try:
            if self.baseImage.size != self.testImage.size: 
                self.result = {self.label: "Failed - images are different sizes"}
                raise StopIteration

            self.diffImage = ImageChops.difference(self.baseImage, 
                    self.testImage)
            self.diffImage.save(self.diff)
            result = False
            for stat in ('stddev', 'mean', 'sum2'):
                for item in getattr(ImageStat.Stat(self.diffImage), stat):
                    if item: 
                        self.result = {self.label: "Failed - see %s" % 
                                self.diff}
                        raise StopIteration
                    else: result = True
        except StopIteration:
            result = False

        if result: self.result = {self.label: "Passed"}

        TC.logger.log(self.result)
        return self.result


class TCNumber(TC):
    """
    Number Comparaison Test Case Class
    """
    def __init__(self):
        TC.__init__(self)
        self.supportedtypes = ("int", "long", "float", "complex", "oct", "hex")

    # Compare 2 numbers by the type provided in the type arg
    def compare(self, label, baseline, undertest, type):
        """
        Compares 2 numbers to see if they are the same. The user may specify
        how to normalize mixed type comparisons via the type argument.
        """
        self.label = label.strip()
        self.baseline = baseline
        self.undertest = undertest
        self.type = type.strip()

        # If we get a valid type, convert to that type and compare
        if self.type in self.supportedtypes:
            # Normalize for comparison
            if self.type == "int":
                self.baseline = int(self.baseline)
                self.undertest = int(self.undertest)
            elif self.type == "long":
                self.baseline = long(self.baseline)
                self.undertest = long(self.undertest)
            elif self.type == "float":
                self.baseline = float(self.baseline)
                self.undertest = float(self.undertest)
            else:
                self.baseline = complex(self.baseline)
                self.undertest = complex(self.undertest)

            #compare
            if self.baseline == self.undertest:
                self.result = {self.label: "Passed - numbers are the same"}
            else:
                self.result = {self.label: "Failed - " + str(self.baseline) + " expected: Got " + str(self.undertest)}
            TC.logger.log(self.result)
            return self.result
        else:
            self.result = {self.label: "Failed - " + self.type + " is not in list of supported types"}
            TC.logger.log(self.result)
            return self.result

class TCBool(TC):
    def __init__(self): pass

    def compare(self, label, _bool):
        """
        If _bool is True, pass.
        If _bool is False, fail.
        """
        if type(_bool) is not bool: raise TypeError
        if _bool: result = {label: "Passed"}
        else: result = {label: "Failed"}
        TC.logger.log(result)

from tree import Node
class TCNode(TC):
    def __init__(self): pass

    def compare(self, label, baseline, undertest):
        """
        If baseline is None, simply check that undertest is a Node.
        If baseline is a Node, check that it is equal to undertest.
        """
        if baseline is not None and not isinstance(baseline, Node): 
            raise TypeError

        if not isinstance(undertest, Node):
            result = {label: "Failed - %s is not a Node" % undertest}
        elif baseline is None:
            result = {label: "Passed - %s is a Node" % undertest}
        elif isinstance(baseline, Node):
            if baseline == undertest: 
                result = {label: "Passed - %s == %s" % (baseline, undertest)}
            else: result = {label: "Failed - %s != %s" % (baseline, undertest)}
        TC.logger.log(result)


if __name__ == '__main__':
    # import the test modules
    import codecs
    from utils import *

    # Set up vars to use to test TC class
    baseline = "test"
    undertest = "test"
    label = "unit test case 1.0"
    encoding = "utf-8"
    result = {}

    # Create the TC instance
    case1 = TC()

    # Fire off the compaison
    result = case1.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - passed
    print(result)

    # Reset variables for failure path
    undertest = "testy"
    encoding = "utf-8"
    result = {}
    label = "unit test case 1.1"

    # Compare again
    result = case1.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - failure
    print(result)

    # Create a TCString instance
    case2 = TCString()

    # Load our variables for this test
    label = " unit test case 2.0"
    encoding = "utf-8"
    baseline = u"groß"
    undertest = u"gro\xdf"
    result = {}

    # Fire off a UTF-8 compare
    result = case2.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - passed
    print(result)

    # Fire the test for ASCII converted to UTF-8 testing
    label = " unit test case 2.1"
    encoding = "utf-8"
    baseline = "please work"
    undertest = "please work"
    result = {}

    result = case2.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - passed
    print(result)

    # Reset variables for an out of range encoding type
    label = " unit test case 2.2"
    encoding = "swahilli"
    baseline = "please work"
    undertest = "please work"
    result = {}

    result = case2.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - Error - not supported
    print(result)

    # Reset variables for unmatched utf-8 strings
    label = " unit test case 2.3"
    encoding = "utf-8"
    baseline = u"groß"
    undertest = "nomatch"
    result = {}

#       result = case2.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - failure
    print(result)

    # Reset variables for inherited TC base compare
    label = " unit test case 2.4"
    baseline = "This is inhereited"
    undertest = "This is inhereited"
    encoding = "utf-8"
    result = {}

    result = case2.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - Passed
    print(result)


    # Include real CJKI (UTF-8) charcters for string compare
    # For this first test case, we are comparing same JA characters
    label = " unit test case 2.5"
    encoding = "utf-8"
    baseline = u"あか"
    undertest = u"あか"
    result = {}

    result = case2.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - Passed
    print(result)


    # Testing different JA characters
    label = " unit test case 2.6"
    encoding = "utf-8"
    baseline = u"あか"
    undertest = u"元気"
    result = {}

    result = case2.compare(label, baseline, undertest, encoding)

    # Print the result - should be label - Failed
    print(result)



    # Test the timestamper class
    # Create a new timestamp instance
    # Print the file format time
    stamp1 = TimeStamp()
    presently = stamp1.fileStamp("filestamp")

    # Print - should be filenameYYYYMMDD with local systems date
    print presently

    # Make a stamp entry
    entry = stamp1.entryStamp()

    # Print the entrystamp - should be YYYY-MM-DD_HH:MM:SS with local system time
    print entry

    # Copmare different colors
    label = "unit test case 3.0"
    baseline = "../examples/data/20w.png"
    undertest = "../examples/data/20b.png"
    result = {}

    # Create a TCImage instance
    case3 = TCImage()

    # Fire off the compare
    result = case3.compare(label, baseline, undertest)

    # Print the result Should be label - Failed
    print result

    # Compare different sizes
    label = "unit test case 3.1"
    baseline = "../examples/data/20w.png"
    undertest = "../examples/data/10w.png"
    result = {}

    # Fire off the compare
    result = case3.compare(label, baseline, undertest)

    # Print the result Should be label - Failed
    print result

    # Compare the same image
    label = "unit test case 3.2"
    baseline = "../examples/data/10w.png"
    undertest = "../examples/data/10w.png"
    result = {}

    # Fire off the compare
    result = case3.compare(label, baseline, undertest)

    # Print the result Should be label - Passed
    print result

    # Number comparison tests
    label = "unit test case 4.0"
    baseline = 42
    undertest = 42
    type = "int"
    result = {}

    # Make a TCNumber instance
    case4 = TCNumber()

    # Make a simple int compare
    result = case4.compare(label, baseline, undertest, type)

    # Should be Passed
    print result

    # Now make the int fail
    label = "unit test case 4.1"
    undertest = 999
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should fail
    print result

    # Now long pass
    label = "unit test case 4.2"
    baseline = 1112223334445556667778889990L
    undertest = 1112223334445556667778889990L
    type = "long"
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should pass
    print result

    # Float Pass
    label = "unit test case 4.3"
    baseline = 99.432670
    undertest = 99.432670
    type = "float"
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should pass
    print result

    # Complex pass
    label = "unit test case 4.4"
    baseline = 67+3j
    undertest = 67+3j
    type = "complex"
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should pass
    print result

    # Octal pass
    label = "unit test case 4.5"
    baseline = 0400
    undertest = 0400
    type = "oct"
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should pass
    print result

    # Hex pass
    label = "unit test case 4.6"
    baseline = 0x100
    undertest = 0x100
    type = "hex"
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should pass
    print result

    # Conversion pass - pass in equivalent hex and octal but compare as int
    label = "unit test case 4.7"
    baseline = 0x100
    undertest = 0400
    type = "int"
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should pass
    print result

    # Give a bogus type
    label = "unit test case 4.8"
    type = "face"
    result = {}

    result = case4.compare(label, baseline, undertest, type)

    # Should fail - unsupported type
    print result



#!/usr/bin/python

import os
import signal
import sys

from dogtail.config import config
config.logDebugToFile = False
from dogtail.procedural import *
from dogtail.tc import TCNode, TCBool
tcn = TCNode()

def appStartup(binary, roleName = "menu item"):
    """Launches the given binary, then checks to see that the application 
    started correctly by looking for a Node with the given roleName."""

    pid = run(binary)
    tcn.compare("app exists", None, focus.application.node)
    focus.widget.node = focus.app.node.child(roleName=roleName)
    tcn.compare("app has a %s" % roleName, None, focus.widget.node)
    os.kill(pid, signal.SIGTERM)

if __name__ == "__main__":
    binary = sys.argv[1]
    appStartup(binary)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unit tests for the dogtail.predicate package
"""
import unittest
import dogtail.tree
import dogtail.predicate


class TestPredicate(unittest.TestCase):
    class DummyNode:
        def __init__(self, name='', roleName='', description=''):
            self.name = name
            self.roleName = roleName
            self.description = description
            self.labeller = None

    def test_capitalization(self):
        self.assertEquals(dogtail.predicate.makeCamel("gnome-terminal"), "gnomeTerminal")
        self.assertEquals(dogtail.predicate.makeCamel("Evolution - Mail"), "evolutionMail")
        self.assertEquals(dogtail.predicate.makeCamel('self.assertEquals(makeCamel("Evolution - Mail"), "evolutionMail")'), "selfAssertequalsMakecamelEvolutionMailEvolutionmail")

    def test_abstract_class(self):
        predicate = dogtail.predicate.Predicate()
        self.assertRaises(NotImplementedError, predicate.satisfiedByNode, None)
        self.assertRaises(NotImplementedError, predicate.makeScriptMethodCall, None)
        self.assertRaises(NotImplementedError, predicate.makeScriptVariableName)
        self.assertRaises(NotImplementedError, predicate.describeSearchResult, None)

    def test_correct_equality(self):
        predicate1 = dogtail.predicate.Predicate()
        predicate2 = dogtail.predicate.Predicate()
        self.assertEquals(predicate1, predicate2)

    def test_incorrect_equality(self):
        predicate = dogtail.predicate.Predicate()
        self.assertNotEquals(predicate, self)

    def test_predicates_application(self):
        dummyApp = self.DummyNode('dummy', 'application')
        appPredicate = dogtail.predicate.IsAnApplicationNamed(dummyApp.name)
        self.assertTrue(appPredicate.satisfiedByNode(dummyApp))
        self.assertEquals(appPredicate.makeScriptMethodCall(True), u'application("dummy")')
        self.assertEquals(appPredicate.makeScriptVariableName(), u'dummyApp')

    def test_predicates_window(self):
        dummyWin = self.DummyNode('dummy', 'frame')
        self.assertTrue(dogtail.predicate.IsAWindow().satisfiedByNode(dummyWin))
        self.assertEquals(dogtail.predicate.IsAWindow().describeSearchResult(), 'window')

    def test_predicates_window_named(self):
        dummyWin = self.DummyNode('dummy', 'frame')
        frameNamedPredicate = dogtail.predicate.IsAWindowNamed(dummyWin.name)
        self.assertTrue(frameNamedPredicate.satisfiedByNode(dummyWin))
        self.assertEquals(frameNamedPredicate.makeScriptMethodCall(False), u'window("dummy")')
        self.assertEquals(frameNamedPredicate.makeScriptVariableName(), u'dummyWin')

    def test_predicates_menu_named(self):
        dummyMenu = self.DummyNode('dummy', 'menu')
        menuNamedPredicate = dogtail.predicate.IsAMenuNamed(dummyMenu.name)
        self.assertTrue(menuNamedPredicate.satisfiedByNode(dummyMenu))
        self.assertEquals(menuNamedPredicate.makeScriptMethodCall(False), u'menu("dummy", recursive=False)')
        self.assertEquals(menuNamedPredicate.makeScriptVariableName(), u'dummyMenu')

    def test_predicates_menu_item_named(self):
        dummyMenuItem = self.DummyNode('dummy', 'menu item')
        menuItemNamedPredicate = dogtail.predicate.IsAMenuItemNamed(dummyMenuItem.name)
        self.assertTrue(menuItemNamedPredicate.satisfiedByNode(dummyMenuItem))
        self.assertEquals(menuItemNamedPredicate.makeScriptMethodCall(False), u'menuItem("dummy", recursive=False)')
        self.assertEquals(menuItemNamedPredicate.makeScriptVariableName(), u'dummyMenuItem')

    def test_predicates_text_entry_named(self):
        dummyText = self.DummyNode('dummy', 'text')
        textNamedPredicate = dogtail.predicate.IsATextEntryNamed(dummyText.name)
        self.assertTrue(textNamedPredicate.satisfiedByNode(dummyText))
        self.assertEquals(textNamedPredicate.makeScriptMethodCall(False), u'textentry("dummy", recursive=False)')
        self.assertEquals(textNamedPredicate.makeScriptVariableName(), u'dummyEntry')

    def test_predicates_button_named(self):
        dummyButton = self.DummyNode('dummy', 'push button')
        buttonNamedPredicate = dogtail.predicate.IsAButtonNamed(dummyButton.name)
        self.assertTrue(buttonNamedPredicate.satisfiedByNode(dummyButton))
        self.assertEquals(buttonNamedPredicate.makeScriptMethodCall(False), u'button("dummy", recursive=False)')
        self.assertEquals(buttonNamedPredicate.makeScriptVariableName(), u'dummyButton')

    def test_predicates_page_tab_named(self):
        dummyTab = self.DummyNode('dummy', 'page tab')
        pageTabNamedPredicate = dogtail.predicate.IsATabNamed(dummyTab.name)
        self.assertTrue(pageTabNamedPredicate.satisfiedByNode(dummyTab))
        self.assertEquals(pageTabNamedPredicate.makeScriptMethodCall(False), u'tab("dummy", recursive=False)')
        self.assertEquals(pageTabNamedPredicate.makeScriptVariableName(), u'dummyTab')

    def test_predicates_generic_by_name(self):
        dn1 = self.DummyNode('dummy name 1', 'dummy role 1', 'dummy desc 1')
        genericPredicateByName = dogtail.predicate.GenericPredicate(name=dn1.name)
        self.assertTrue(genericPredicateByName.satisfiedByNode(dn1))
        self.assertEquals(genericPredicateByName.makeScriptMethodCall(False), u'child( name="dummy name 1", recursive=False)')
        self.assertEquals(genericPredicateByName.makeScriptVariableName(), u'dummyName1Node')

    def test_predicates_generic_by_label(self):
        dn1 = self.DummyNode('dummy name 1', 'dummy role 1', 'dummy desc 1')
        genericPredicateByRole = dogtail.predicate.GenericPredicate(roleName=dn1.roleName)
        self.assertTrue(genericPredicateByRole.satisfiedByNode(dn1))
        self.assertEquals(genericPredicateByRole.makeScriptMethodCall(False), u"child( roleName='dummy role 1', recursive=False)")
        self.assertEquals(genericPredicateByRole.makeScriptVariableName(), u'dummyRole1Node')

    def test_predicates_generic_by_description(self):
        dn1 = self.DummyNode('dummy name 1', 'dummy role 1', 'dummy desc 1')
        genericPredicateByDescription = dogtail.predicate.GenericPredicate(description=dn1.description)
        self.assertTrue(genericPredicateByDescription.satisfiedByNode(dn1))
        self.assertEquals(genericPredicateByDescription.makeScriptMethodCall(False), u"child( description='dummy desc 1', recursive=False)")
        self.assertEquals(genericPredicateByDescription.makeScriptVariableName(), u'dummyDesc1Node')

    def test_predicates_named(self):
        dn1 = self.DummyNode('dummy name 1', 'dummy role 1', 'dummy desc 1')
        genericNamedPredicate = dogtail.predicate.IsNamed(dn1.name)
        self.assertTrue(genericNamedPredicate.satisfiedByNode(dn1))
        self.assertEquals(genericNamedPredicate.makeScriptMethodCall(False), u'child(name="dummy name 1", recursive=False)')
        self.assertEquals(genericNamedPredicate.makeScriptVariableName(), u'dummyName1Node')

    def test_predicates_labelled_as(self):
        dn1 = self.DummyNode('dummy name 1', 'dummy role 1', 'dummy desc 1')
        dn2 = self.DummyNode('dummy name 2', 'dummy role 2', 'dummy desc 2')
        dn2.labeller = dn1
        genericLabelledPredicate = dogtail.predicate.IsLabelledAs(dn1.name)
        self.assertTrue(genericLabelledPredicate.satisfiedByNode(dn2))
        self.assertFalse(genericLabelledPredicate.satisfiedByNode(dn1))
        self.assertEquals(genericLabelledPredicate.makeScriptMethodCall(False), u'child(label="dummy name 1", recursive=False)')
        self.assertEquals(genericLabelledPredicate.makeScriptVariableName(), u'dummyName1Node')

if __name__ == '__main__':
    unittest.main()



"""
The configuration module.
"""
__author__ = "Zack Cerza <zcerza@redhat.com>, David Malcolm <dmalcolm@redhat.com>"

import os
import sys
import locale


def _userTmpDir(baseName):
    # i.e. /tmp/dogtail-foo
    return '-'.join(('/'.join(('/tmp', baseName)), os.getlogin()))

class _Config(object):
    """
    Contains configuration parameters for the dogtail run.

    scratchDir(str):
    Directory where things like screenshots are stored.

    dataDir(str):
    Directory where related data files are located.

    logDir(str):
    Directory where dogtail.tc.TC*-generated logs are stored.

    scriptName(str) [Read-Only]:
    The name of the script being run.

    encoding(str)
    The encoding for text, used by dogtail.tc.TCString .

    actionDelay(float):
    The delay after an action is executed.

    typingDelay(float):
    The delay after a character is typed on the keyboard.

    runInterval(float):
    The interval at which dogtail.utils.run() and dogtail.procedural.run()
    check to see if the application has started up.

    runTimeout(int):
    The timeout after which dogtail.utils.run() and dogtail.procedural.run()
    give up on looking for the newly-started application.

    searchBackoffDuration (float):
    Time in seconds for which to delay when a search fails.

    searchWarningThreshold (int):
    Number of retries before logging the individual attempts at a search.

    searchCutoffCount (int):
    Number of times to retry when a search fails.

    defaultDelay (float):
    Default time in seconds to sleep when delaying.

    childrenLimit (int):
    When there are a very large number of children of a node, only return
    this many, starting with the first.

    debugSearching (boolean):
    Whether to write info on search backoff and retry to the debug log.

    debugSleep (boolean):
    Whether to log whenever we sleep to the debug log.

    debugSearchPaths (boolean):
    Whether we should write out debug info when running the SearchPath
    routines.

    absoluteNodePaths (boolean):
    Whether we should identify nodes in the logs with long 'abcolute paths', or
    merely with a short 'relative path'. FIXME: give examples

    ensureSensitivity (boolean):
    Should we check that ui nodes are sensitive (not 'greyed out') before
    performing actions on them? If this is True (the default) it will raise
    an exception if this happens. Can set to False as a workaround for apps
    and toolkits that don't report sensitivity properly.

    debugTranslation (boolean):
    Whether we should write out debug information from the translation/i18n
    subsystem.

    blinkOnActions (boolean):
    Whether we should blink a rectangle around a Node when an action is
    performed on it.

    fatalErrors (boolean):
    Whether errors encountered in dogtail.procedural should be considered
    fatal. If True, exceptions will be raised. If False, warnings will be
    passed to the debug logger.

    checkForA11y (boolean):
    Whether to check if accessibility is enabled. If not, just assume it is
    (default True).

    logDebugToFile (boolean):
    Whether to write debug output to a log file.

    logDebugToStdOut (boolean):
    Whether to print log output to console or not (default True).
    """
    @property
    def scriptName(self):
        return os.path.basename(sys.argv[0]).replace('.py','')

    @property
    def encoding(self):
        return locale.getpreferredencoding().lower()

    defaults = {
            # Storage
            'scratchDir' : '/'.join((_userTmpDir('dogtail'), '')),
            'dataDir' : '/'.join((_userTmpDir('dogtail'), 'data', '')),
            'logDir' : '/'.join((_userTmpDir('dogtail'), 'logs', '')),
            'scriptName' : scriptName.fget(None),
            'encoding' : encoding.fget(None),
            'configFile' : None,
            'baseFile' : None,

            # Timing and Limits
            'actionDelay' : 1.0,
            'typingDelay' : 0.075,
            'runInterval' : 0.5,
            'runTimeout' : 30,
            'searchBackoffDuration' : 0.5,
            'searchWarningThreshold' : 3,
            'searchCutoffCount' : 20,
            'defaultDelay' : 0.5,
            'childrenLimit' : 100,

            # Debug
            'debugSearching' : False,
            'debugSleep' : False,
            'debugSearchPaths' : False,
            'logDebugToStdOut' : True,
            'absoluteNodePaths' : False,
            'ensureSensitivity' : False,
            'debugTranslation' : False,
            'blinkOnActions' : False,
            'fatalErrors' : False,
            'checkForA11y' : True,

            # Logging
            'logDebugToFile' : True
    }

    options = {}

    invalidValue = "__INVALID__"

    def __init__(self):
        _Config.__createDir(_Config.defaults['scratchDir'])
        _Config.__createDir(_Config.defaults['logDir'])
        _Config.__createDir(_Config.defaults['dataDir'])

    def __setattr__(self, name, value):
        if name not in config.defaults:
            raise AttributeError, name + " is not a valid option."

        elif _Config.defaults[name] != value or \
                _Config.options.get(name, _Config.invalidValue) != value:
            if 'Dir' in name:
                _Config.__createDir(value)
                if value[-1] != os.path.sep: value = value + os.path.sep
            elif name == 'logDebugToFile':
                import logging
                logging.debugLogger = logging.Logger('debug', value)
            _Config.options[name] = value

    def __getattr__(self, name):
        try: return _Config.options[name]
        except KeyError:
            try: return _Config.defaults[name]
            except KeyError: raise AttributeError, name + \
                    " is not a valid option."

    def __createDir(cls, dirName, perms = 0777):
        """
        Creates a directory (if it doesn't currently exist), creating any
        parent directories it needs.

        If perms is None, create with python's default permissions.
        """
        dirName = os.path.abspath(dirName)
        #print "Checking for %s ..." % dirName,
        if not os.path.isdir(dirName):
            if perms:
                umask = os.umask(0)
                os.makedirs(dirName, perms)
                os.umask(umask)
            else: os.makedirs(dirName)
    __createDir = classmethod(__createDir)

    def load(self, dict):
        """
        Loads values from dict, preserving any options already set that are not overridden.
        """
        _Config.options.update(dict)

    def     reset(self):
        """
        Resets all settings to their defaults.
        """
        _Config.options = {}


config = _Config()

if __name__ == '__main__':
    anyFailed = False
    def failOrPass(failure, description):
        if failure:
            anyFailed = True
            print "FAILED: " + description
        else: print "PASSED: " + description

    # BEGIN tests

    failure = False
    for option in config.defaults.keys():
        failure = failure or not (getattr(config, option) == \
                config.defaults[option])
        print failure, option, getattr(config, option), config.defaults[option]
    failOrPass(failure, "Reading all default values")

    failure = True
    failure = config.ensureSensitivity != config.defaults['ensureSensitivity']
    config.ensureSensitivity = False
    failure = failure or config.ensureSensitivity == True
    config.ensureSensitivity = True
    failure = failure or config.ensureSensitivity != True
    failOrPass(failure, "Setting ensureSensitivity")

    failure = True
    failure = not os.path.isdir(config.defaults['scratchDir'])
    failure = failure or not os.path.isdir(config.defaults['logDir'])
    failure = failure or not os.path.isdir(config.defaults['dataDir'])
    failOrPass(failure, "Looking for default directories")

    failure = True
    config.scratchDir = '/tmp/dt'
    failure = not os.path.isdir('/tmp/dt')
    config.logDir = '/tmp/dt_log/'
    failure = failure or not os.path.isdir('/tmp/dt_log/')
    config.dataDir = '/tmp/dt_data'
    failure = failure or not os.path.isdir('/tmp/dt_data')
    failOrPass(failure, "Changing default directories")

    # END tests

    if anyFailed: sys.exit(1)



#!/usr/bin/env python
# Dogtail demo script

from dogtail.config import config
#config.debugSleep = True
#config.debugSearching = True
#config.debugTranslation = True

import dogtail.tc
from dogtail.procedural import *
from dogtail.utils import screenshot
from dogtail.predicate import GenericPredicate

# These next two lines get us translations for free. To see the script run
# translated, run it like this:
#  LANG=ja_JP.UTF-8 ./gedit-test-utf8-procedural-api.py
# You might also want to set config.debugTranslation and
# config.debugSearching to True, just for fun.
#import dogtail.i18n
#dogtail.i18n.loadTranslationsFromPackageMoFiles('gedit')

from os import environ, path, remove

# Load our persistent Dogtail objects
TestString = dogtail.tc.TCString()

# Remove the output file, if it's still there from a previous run
if path.isfile(path.join(path.expandvars("$HOME"), "Desktop", "UTF8demo.txt")):
    remove(path.join(path.expandvars("$HOME"), "Desktop", "UTF8demo.txt"))

# Start gedit.
run('gedit')

# Set focus on gedit
focus.application('gedit')

# Focus gedit's text buffer.
focus.text()

# Load the UTF-8 demo file. Use codecs.open() instead of open().
from codecs import open
from sys import path
utfdemo = open(path[0] + '/data/UTF-8-demo.txt')

# Load the UTF-8 demo file into the text buffer.
focus.widget.text = utfdemo.read()

# Take a screenshot of the window
#screenshot()

# Click gedit's Save button.
click.button('Save')

# Focus gedit's Save As... dialog
try:
    focus.widget.findByPredicate(GenericPredicate(roleName='file chooser'))
except FocusError:
    try:
        # This string changed somewhere around gedit 2.13.2.
        # This is the new string
        focus.dialog(u'Save As\u2026')
    except FocusError:
        # Fall back to the old string.
        focus.dialog('Save as...')

# Click the Desktop widget
click('Desktop', roleName = 'table cell')

# Focus on dialog again
try:
    focus.widget.findByPredicate(GenericPredicate(roleName='file chooser'))
except FocusError:
    try:
        # This string changed somewhere around gedit 2.13.2.
        # This is the new string
        focus.dialog(u'Save As\u2026')
    except FocusError:
        # Fall back to the old string.
        focus.dialog('Save as...')
# We want to save to the file name 'UTF8demo.txt'.
focus.text()
focus.widget.text = 'UTF8demo.txt'


# And focus on dialog again
try:
    focus.widget.findByPredicate(GenericPredicate(roleName='file chooser'))
except FocusError:
    try:
        # This string changed somewhere around gedit 2.13.2.
        # This is the new string
        focus.dialog(u'Save As\u2026')
    except FocusError:
        # Fall back to the old string.
        focus.dialog('Save as...')

# Click the Save button.
click('Save')

# Let's quit now.
click('File')
click('Quit')

# We have driven gedit now lets check to see if the saved file is the same as
# the baseline file

# Read in the "gold" file
import codecs
try:
    # When reading the file, we have to make sure and tell codecs.open() which
    # encoding we're using, otherwise python gets confused later.
    gold = open(path[0] + '/data/UTF-8-demo.txt', encoding='utf-8').readlines()
except IOError:
    print "File open failed"

# Read the test file for comparison
filepath = environ['HOME'] + '/Desktop/UTF8demo.txt'
# When reading the file, we have to make sure and tell codecs.open() which
# encoding we're using, otherwise python gets confused later.
testfile = open(filepath, encoding='utf-8').readlines()

# We now have the original and saved files as lists. Let's compare them line
# by line to see if they are the same
i = 0
for baseline in gold:
    label = "line test " + str(i + 1)
    TestString.compare(label, baseline, testfile[i], encoding='utf-8')
    i = i + 1



#!/usr/bin/env python
__author__ = "Zack Cerza <zcerza@redhat.com>"

from distutils.core import setup
from distutils.command.bdist_rpm import bdist_rpm

def examples():
    import os
    exList = os.listdir(os.curdir + '/examples/')
    result = []
    for ex in exList:
        if ex.split('.')[-1] == 'py':
            result = result + ['examples/' + ex]
    return result

def examples_data():
    import os
    dataList = os.listdir(os.curdir + '/examples/data/')
    result = []
    for data in dataList:
		result = result + ['examples/data/' + data]
    return result

def tests():
    import os
    exList = os.listdir(os.curdir + '/tests/')
    result = []
    for ex in exList:
        if ex.split('.')[-1] == 'py':
            result = result + ['tests/' + ex]
    return result

def sniff_icons():
    import os
    list = os.listdir(os.curdir + '/sniff/icons/')
    result = []
    for file in list:
        if file.split('.')[-1] in ('xpm'):
            result = result + ['sniff/icons/' + file]
    return result

def icons(ext_tuple):
    import os
    list = os.listdir(os.curdir + '/icons/')
    result = []
    for file in list:
        if file.split('.')[-1] in ext_tuple:
            result = result + ['icons/' + file]
    return result

def scripts():
    import os
    list = os.listdir(os.curdir + '/scripts/')
    result = ['sniff/sniff']
    for file in list:
		result = result + ['scripts/' + file]
    return result

def session_file():
    result = ['scripts/gnome-dogtail-headless.session']
    return result

setup (
        name = 'dogtail',
        version = '0.8.2',
        description = """GUI test tool and automation framework that uses Accessibility (a11y) technologies to communicate with desktop applications.""",
        author = """Zack Cerza <zcerza@redhat.com>,
Ed Rousseau <rousseau@redhat.com>,
David Malcolm <dmalcolm@redhat.com>
Vitezslav Humpa <vhumpa@redhat.com>""",
        author_email = 'dogtail-list@gnome.org',
        url = 'http://dogtail.fedorahosted.org/',
        packages = ['dogtail'],
        scripts = scripts(),
        data_files = [
                                ('share/doc/dogtail/examples',
                                        examples() ),
                                ('share/doc/dogtail/examples/data',
                                        examples_data() ),
                                ('share/doc/dogtail/tests',
                                        tests() ),
                                ('share/dogtail/glade', ['sniff/sniff.ui']),
                                ('share/dogtail/icons', sniff_icons() ),
                                ('share/applications', ['sniff/sniff.desktop']),
                                ('share/icons/hicolor/48x48/apps', icons('png')),
                                ('share/icons/hicolor/scalable/apps', icons('svg'))
                                ],
        cmdclass = {
                'bdist_rpm': bdist_rpm
                }
)

# vim: sw=4 ts=4 sts=4 noet ai



# -*- coding: utf-8 -*-
"""
Handles raw input using AT-SPI event generation.

Note: Think of keyvals as keysyms, and keynames as keystrings.

Authors: David Malcolm <dmalcolm@redhat.com>, Zack Cerza <zcerza@redhat.com>
"""

__author__ = """
David Malcolm <dmalcolm@redhat.com>,
Zack Cerza <zcerza@redhat.com>
"""
import gi

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

import sys
from gi.repository import Gtk
from gi.repository import Gdk
from config import config
from utils import doDelay
from logging import debugLogger as logger
from pyatspi import Registry as registry
from pyatspi import (KEY_SYM, KEY_PRESS, KEY_PRESSRELEASE, KEY_RELEASE)

def doTypingDelay():
    doDelay(config.typingDelay)

def click (x, y, button = 1):
    """
    Synthesize a mouse button click at (x,y)
    """
    logger.log("Mouse button %s click at (%s,%s)"%(button,x,y))
    registry.generateMouseEvent(x, y, 'b%sc' % button)
    doDelay(config.actionDelay)

def doubleClick (x, y, button = 1):
    """
    Synthesize a mouse button double-click at (x,y)
    """
    logger.log("Mouse button %s doubleclick at (%s,%s)"%(button,x,y))
    registry.generateMouseEvent(x,y, 'b%sd' % button)
    doDelay()

def press (x, y, button = 1):
    """
    Synthesize a mouse button press at (x,y)
    """
    logger.log("Mouse button %s press at (%s,%s)"%(button,x,y))
    registry.generateMouseEvent(x,y, 'b%sp' % button)
    doDelay()

def release (x, y, button = 1):
    """
    Synthesize a mouse button release at (x,y)
    """
    logger.log("Mouse button %s release at (%s,%s)"%(button,x,y))
    registry.generateMouseEvent(x,y, 'b%sr' % button)
    doDelay()

def absoluteMotion (x, y, mouseDelay=None):
    """
    Synthesize mouse absolute motion to (x,y)
    """
    logger.log("Mouse absolute motion to (%s,%s)"%(x,y))
    registry.generateMouseEvent(x,y, 'abs')
    if mouseDelay:
        doDelay(mouseDelay)
    else:
        doDelay()

def relativeMotion (x, y, mouseDelay=None):
    logger.log("Mouse relative motion of (%s,%s)"%(x,y))
    registry.generateMouseEvent(x,y, 'rel')
    if mouseDelay:
        doDelay(mouseDelay)
    else:
        doDelay()

def drag(fromXY, toXY, button = 1):
    """
    Synthesize a mouse press, drag, and release on the screen.
    """
    logger.log("Mouse button %s drag from %s to %s"%(button, fromXY, toXY))

    (x,y) = fromXY
    press (x, y, button)
    #doDelay()

    (x,y) = toXY
    absoluteMotion(x,y)
    doDelay()

    release (x, y, button)
    doDelay()

def typeText(string):
    """
    Types the specified string, one character at a time.
    """
    if not isinstance(string, unicode):
        string = string.decode('utf-8')
    for char in string:
        pressKey(char)

keyNameAliases = {
    'enter' : 'Return',
    'esc' : 'Escape',
    'alt' : 'Alt_L',
    'control' : 'Control_L',
    'ctrl' : 'Control_L',
    'shift' : 'Shift_L',
    'del' : 'Delete',
    'ins' : 'Insert',
    'pageup' : 'Page_Up',
    'pagedown' : 'Page_Down',
    ' ' : 'space',
    '\t' : 'Tab',
    '\n' : 'Return'
}

def keySymToUniChar(keySym):
    i = Gdk.keyval_to_unicode(keySym)
    if i: UniChar = unichr(i)
    else: UniChar = ''
    return UniChar

def uniCharToKeySym(uniChar):
    # OK, if it's not actually unicode we can fix that, right?
    if not isinstance(uniChar, unicode): uniChar = unicode(uniChar)
    i = ord(uniChar)
    keySym = Gdk.unicode_to_keyval(i)
    return keySym

def keySymToKeyName(keySym):
    return Gdk.keyval_name(keySym)

def keyNameToKeySym(keyName):
    try:
        keyName = keyNameAliases.get(keyName.lower(), keyName)
        keySym = Gdk.keyval_from_name(keyName)
        if not keySym: keySym = getattr(Gdk, keyName)
    except AttributeError:
        try: keySym = uniCharToKeySym(keyName)
        except TypeError: raise KeyError, keyName
    return keySym

def keyNameToKeyCode(keyName):
    """
    Use GDK to get the keycode for a given keystring.

    Note that the keycode returned by this function is often incorrect when
    the requested keystring is obtained by holding down the Shift key.

    Generally you should use uniCharToKeySym() and should only need this
    function for nonprintable keys anyway.
    """
    keymap = Gdk.Keymap.get_for_display(Gdk.Display.get_default())
    entries = keymap.get_entries_for_keyval( \
                Gdk.keyval_from_name(keyName))
    try: return entries[1][0].keycode
    except TypeError: pass

def pressKey(keyName):
    """
    Presses (and releases) the key specified by keyName.
    keyName is the English name of the key as seen on the keyboard. Ex: 'enter'
    Names are looked up in Gdk.KEY_ If they are not found there, they are
    looked up by uniCharToKeySym().
    """
    keySym = keyNameToKeySym(keyName)
    registry.generateKeyboardEvent(keySym, None, KEY_SYM)
    doTypingDelay()

def keyCombo(comboString):
    """
    Generates the appropriate keyboard events to simulate a user pressing the
    specified key combination.

    comboString is the representation of the key combo to be generated.
    e.g. '<Control><Alt>p' or '<Control><Shift>PageUp' or '<Control>q'
    """
    strings = []
    for s in comboString.split('<'):
        if s:
            for S in s.split('>'):
                if S:
                    S = keyNameAliases.get(S.lower(), S)
                    strings.append(S)
    for s in strings:
        if not hasattr(Gdk, s):
            if not hasattr(Gdk, 'KEY_' + s):
                raise ValueError, "Cannot find key %s" % s
    modifiers = strings[:-1]
    finalKey = strings[-1]
    for modifier in modifiers:

        code = keyNameToKeyCode(modifier)
        registry.generateKeyboardEvent(code, None, KEY_PRESS)
    code = keyNameToKeyCode(finalKey)
    registry.generateKeyboardEvent(code, None, KEY_PRESSRELEASE)
    for modifier in modifiers:
        code = keyNameToKeyCode(modifier)
        registry.generateKeyboardEvent(code, None, KEY_RELEASE)
    doDelay()




# -*- coding: utf-8 -*-
"""
Various utilities

Authors: Ed Rousseau <rousseau@redhat.com>, Zack Cerza <zcerza@redhat.com, David Malcolm <dmalcolm@redhat.com>
"""

__author__ = """Ed Rousseau <rousseau@redhat.com>,
Zack Cerza <zcerza@redhat.com,
David Malcolm <dmalcolm@redhat.com>
"""

import os
import sys
import subprocess
import re
import cairo
from gi.repository import Gtk
from gi.repository import GObject
from config import config
from time import sleep
from logging import debugLogger as logger
from logging import TimeStamp
from errors import DependencyNotFoundError

def screenshot(file = 'screenshot.png', timeStamp = True):
    """
    This function wraps the ImageMagick import command to take a screenshot.

    The file argument may be specified as 'foo', 'foo.png', or using any other
    extension that ImageMagick supports. PNG is the default.

    By default, screenshot filenames are in the format of foo_YYYYMMDD-hhmmss.png .
    The timeStamp argument may be set to False to name the file foo.png.
    """
    if not isinstance(timeStamp, bool):
        raise TypeError, "timeStampt must be True or False"
    # config is supposed to create this for us. If it's not there, bail.
    assert os.path.isdir(config.scratchDir)

    baseName = ''.join(file.split('.')[0:-1])
    fileExt = file.split('.')[-1].lower()
    if not baseName:
        baseName = file
        fileExt = 'png'

    if timeStamp:
        ts = TimeStamp()
        newFile = ts.fileStamp(baseName) + '.' + fileExt
        path = config.scratchDir + newFile
    else:
        newFile = baseName + '.' + fileExt
        path = config.scratchDir + newFile

    from gi.repository import Gdk
    from gi.repository import GObject
    from gi.repository import GdkPixbuf
    rootWindow = Gdk.get_default_root_window()
    geometry = rootWindow.get_geometry()
    pixbuf = GdkPixbuf.Pixbuf(colorspace=GdkPixbuf.Colorspace.RGB,
                              has_alpha=False,
                              bits_per_sample=8,
                              width=geometry[2],
                              height=geometry[3])

    pixbuf = Gdk.pixbuf_get_from_window(rootWindow, 0, 0, \
                                        geometry[2], geometry[3])
    # GdkPixbuf.Pixbuf.save() needs 'jpeg' and not 'jpg'
    if fileExt == 'jpg': fileExt = 'jpeg'
    try:
        pixbuf.savev(path, fileExt, [], [])
    except GObject.GError:
        raise ValueError, "Failed to save screenshot in %s format" % fileExt
    assert os.path.exists(path)
    logger.log("Screenshot taken: " + path)
    return path

def run(string, timeout=config.runTimeout, interval=config.runInterval, desktop=None, dumb=False, appName=''):
    """
    Runs an application. [For simple command execution such as 'rm *', use os.popen() or os.system()]
    If dumb is omitted or is False, polls at interval seconds until the application is finished starting, or until timeout is reached.
    If dumb is True, returns when timeout is reached.
    """
    if not desktop: from tree import root as desktop
    args = string.split()
    name = args[0]
    os.environ['GTK_MODULES'] = 'gail:atk-bridge'
    pid = subprocess.Popen(args, env = os.environ).pid

    if not appName:
        appName=args[0]

    if dumb:
        # We're starting a non-AT-SPI-aware application. Disable startup detection.
        doDelay(timeout)
    else:
        # Startup detection code
        # The timing here is not totally precise, but it's good enough for now.
        time = 0
        while time < timeout:
            time = time + interval
            try:
                for child in desktop.children[::-1]:
                    if child.name == appName:
                        for grandchild in child.children:
                            if grandchild.roleName == 'frame':
                                from procedural import focus
                                focus.application.node = child
                                doDelay(interval)
                                return pid
            except AttributeError: pass
            doDelay(interval)
    return pid

def doDelay(delay=None):
    """
    Utility function to insert a delay (with logging and a configurable
    default delay)
    """
    if delay is None:
        delay = config.defaultDelay
    if config.debugSleep:
        logger.log("sleeping for %f" % delay)
    sleep(delay)

class Highlight (Gtk.Window):

    def __init__(self, x, y, w, h):
        super(Highlight, self).__init__()
        self.set_decorated(False)
        self.set_has_resize_grip(False)
        self.set_default_size(w, h)
        self.screen = self.get_screen()
        self.visual = self.screen.get_rgba_visual()
        if self.visual is not None and self.screen.is_composited():
            self.set_visual(self.visual)
        self.set_app_paintable(True)
        self.connect("draw", self.area_draw)
        self.show_all()
        self.move(x,y)

    def area_draw(self, widget, cr):
        cr.set_source_rgba(.0, .0, .0, 0.0)
        cr.set_operator(cairo.OPERATOR_SOURCE)
        cr.paint()
        cr.set_operator(cairo.OPERATOR_OVER)
        cr.set_source_rgb(0.9, 0.1, 0.1)
        cr.set_line_width(6)
        cr.rectangle(0, 0, self.get_size()[0], self.get_size()[1])
        cr.stroke()

class Blinker(object):
    INTERVAL_MS = 1000
    main_loop = GObject.MainLoop()
    def __init__(self, x, y, w, h):
        from gi.repository import Gdk
        self.highlight_window = Highlight(x, y, w, h)
        if self.highlight_window.screen.is_composited() is not False:
            self.timeout_handler_id = GObject.timeout_add (Blinker.INTERVAL_MS, self.destroyHighlight)
            self.main_loop.run()
        else:
            self.highlight_window.destroy()

    def destroyHighlight(self):
        self.highlight_window.destroy()
        self.main_loop.quit()
        return False
        
class Lock(object):
    """
    A mutex implementation that uses atomicity of the mkdir operation in UNIX-like
    systems. This can be used by scripts to provide for mutual exlusion, either in single
    scripts using threads etc. or i.e. to handle sitations of possible collisions among
    multiple running scripts. You can choose to make randomized single-script wise locks
    or a more general locks if you do not choose to randomize the lockdir name
    """
    def __init__(self, location='/tmp', lockname='dogtail_lockdir_', randomize=True):
        """
        You can change the default lockdir location or name. Setting randomize to
        False will result in no random string being appened to the lockdir name.
        """
        self.lockdir = os.path.join(os.path.normpath(location), lockname)
        if randomize:
            self.lockdir = "%s%s" % (self.lockdir,self.__getPostfix())

    def lock(self):
        """
        Creates a lockdir based on the settings on Lock() instance creation.
        Raises OSError exception of the lock is already present. Should be
        atomic on POSIX compliant systems. 
        """
        locked_msg = 'Dogtail lock: Already locked with the same lock'
        if not os.path.exists(self.lockdir):
            try:
                os.mkdir(self.lockdir)
                return self.lockdir
            except OSError, e:
                if e.errno == errno.EEXIST and os.path.isdir(self.lockdir):
                    raise OSError(locked_msg)
        else:
            raise OSError(locked_msg)

    def unlock(self):
        """
        Removes a lock. Will raise OSError exception if the lock was not present.
        Should be atomic on POSIX compliant systems.
        """
        import os #have to import here for situations when executed from __del__
        if os.path.exists(self.lockdir):
            try:
                os.rmdir(self.lockdir)
            except OSError, e:
                if e.erron == errno.EEXIST:
                    raise OSError('Dogtail unlock: lockdir removed elsewhere!')
        else:
            raise OSError('Dogtail unlock: not locked')

    def __del__(self):
        """
        Makes sure lock is removed when the process ends. Although not when killed indeed.
        """
        self.unlock()

    def __getPostfix(self):
        import random
        import string
        return ''.join(random.choice(string.letters + string.digits) for x in range(5))
        

a11yDConfKey = 'org.gnome.desktop.interface'

def isA11yEnabled():
    """
    Checks if accessibility is enabled via DConf.
    """
    from gi.repository.Gio import Settings
    InterfaceSettings = Settings(a11yDConfKey)
    dconfEnabled = InterfaceSettings.get_boolean('toolkit-accessibility')
    if os.environ.get('GTK_MODULES','').find('gail:atk-bridge') == -1:
        envEnabled = False
    else: envEnabled = True
    return (dconfEnabled or envEnabled)

def bailBecauseA11yIsDisabled():
    if sys.argv[0].endswith("pydoc"): return
    try:
        if file("/proc/%s/cmdline" % os.getpid()).read().find('epydoc') != -1:
            return
    except: pass
    logger.log("Dogtail requires that Assistive Technology support be enabled. Aborting...")
    sys.exit(1)

def enableA11y(enable=True):
    """
    Enables accessibility via DConf.
    """
    from gi.repository.Gio import Settings
    InterfaceSettings = Settings(a11yDConfKey)
    dconfEnabled = InterfaceSettings.set_boolean('toolkit-accessibility', enable)

def checkForA11y():
    """
    Checks if accessibility is enabled, and halts execution if it is not.
    """
    if not isA11yEnabled(): bailBecauseA11yIsDisabled()

def checkForA11yInteractively():
    """
    Checks if accessibility is enabled, and presents a dialog prompting the
    user if it should be enabled if it is not already, then halts execution.
    """
    if isA11yEnabled(): return
    from gi.repository import Gtk
    dialog = Gtk.Dialog('Enable Assistive Technology Support?',
                     None,
                     Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                     (Gtk.STOCK_QUIT, Gtk.ResponseType.CLOSE,
                         "_Enable", Gtk.ResponseType.ACCEPT))
    question = """Dogtail requires that Assistive Technology Support be enabled for it to function. Would you like to enable Assistive Technology support now?

Note that you will have to log out for the change to fully take effect.
    """.strip()
    dialog.set_default_response(Gtk.ResponseType.ACCEPT)
    questionLabel = Gtk.Label(label=question)
    questionLabel.set_line_wrap(True)
    dialog.vbox.pack_start(questionLabel, True, True, 0)
    dialog.show_all()
    result = dialog.run()
    if result == Gtk.ResponseType.ACCEPT:
        logger.log("Enabling accessibility...")
        enableA11y()
    elif result == Gtk.ResponseType.CLOSE:
       bailBecauseA11yIsDisabled()
    dialog.destroy()

